from dataclasses import dataclass
from pylab import plt, mpl, np
from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes, mark_inset
from collections import OrderedDict as OD
import pandas as pd
import os
mpl.rcParams['legend.fontsize'] = 10
mpl.rcParams['font.family'] = ['serif'] # default is sans-serif
mpl.rcParams['font.serif'] = ['Times New Roman']
font = {'family':'Times New Roman', 'weight':'normal', 'size':10}

# this changes font for ticks for my TEC-ISMB paper figures but has no effect here, why?
mpl.rcParams['mathtext.fontset'] = 'stix'
mpl.rcParams['font.family'] = ['STIXGeneral', 'Times New Roman']
mpl.rcParams['font.size'] = 14.0
mpl.rcParams['legend.fontsize'] = 12.5

# plt.style.use('ggplot') 
# plt.style.use('grayscale') # print plt.style.available # get [u'dark_background', u'bmh', u'grayscale', u'ggplot', u'fivethirtyeight']
mpl.style.use('classic')


######################
# Data
def load_dataframe(path='./', index=0):

    df_list = []
    fname_list = []
    for fname in os.listdir(path):
        if '.dat' in fname and 'info' not in fname and 'pdf' not in fname:
            print('--------------', fname)
            fname_list.append(fname)
            df = pd.read_csv(path+fname, na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
            df_list.append(df)

    # index = 0
    df    = df_list[index]
    fname = fname_list[index]
    no_samples = df.shape[0]
    no_traces  = df.shape[1]
    df_info = pd.read_csv(path+"info.dat", na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
    time = np.arange(1,no_samples+1,1) * float(df_info['DOWN_SAMPLE']) * float(df_info['CL_TS'])  # sampling frequency is 10e3
    print('---- List of Keys:')
    for key in df.keys():
        print('\t', key)

    return df_list, time, fname

######################
# Plotting
def locallyZoomIn(ax, data_zoomed, ylim=None, loc1=None, loc2=None, y_axis_shrink=None, **kwarg):
    # extent = [-3, 4, -4, 3]

    if(y_axis_shrink!=None):
        temp_tuple = (ylim[0]*y_axis_shrink, ylim[1]*y_axis_shrink)
        ylim = temp_tuple
        for ind, el in enumerate(data_zoomed):
            temp_tuple = (el[0], el[1]*y_axis_shrink)
            data_zoomed[ind] = temp_tuple

    axins = zoomed_inset_axes(ax, **kwarg) # zoom = 
    # axins.plot(Z2, extent=extent, interpolation="nearest", origin="lower")
    for ind, el in enumerate(data_zoomed):
        if ind==1:
            if len(data_zoomed)>1:
                axins.plot(el[0],el[1], 'r-', alpha=0.7, linewidth=0.8) # '--'
            else:
                axins.plot(el[0],el[1], 'k-', alpha=0.7, lw=0.8)
        else:
            # axins.plot(*el, lw=2)
            axins.plot(el[0],el[1], lw=1)

    # sub region of the original image
    # x1, x2, y1, y2 = 1.2*pi, 1.25*pi, -0.78, -0.54
    # axins.set_xlim(x1, x2)
    # axins.set_ylim(y1, y2)

    if ylim!=None:
        axins.set_ylim(ylim)

    plt.xticks(np.arange(  axins.viewLim.intervalx[0],
                        axins.viewLim.intervalx[1],
                        axins.viewLim.width/2), visible=False)
    plt.yticks(np.arange(  axins.viewLim.intervaly[0],
                        axins.viewLim.intervaly[1],
                        axins.viewLim.height/4), visible=False)

    if(y_axis_shrink==None):
        # draw a bbox of the region of the inset axes in the parent axes and
        # connecting lines between the bbox and the inset axes area
        if loc1!=None and loc2!=None:
            mark_inset(ax, axins, loc1=loc1, loc2=loc2, fc="none", ec="0.5")
        else:
            mark_inset(ax, axins, loc1=2, loc2=3, fc="none", ec="0.5")
        plt.draw()
    return axins
def plot_it(ax, time, ylabel, OrDi, bool_legend=False):
    cnt_temp = 0
    for k, v in OrDi.items():
        cnt_temp += 1
        if cnt_temp == 2:
            ax.plot(time, v, 'r-', alpha=0.7, lw=0.8)
        elif cnt_temp == 3:
            print('A thrid trace is plotted in blue')
            ax.plot(time, v, 'b-', alpha=0.7, lw=0.8)
        else:
            ax.plot(time, v, 'k-', alpha=0.7, lw=0.8) # label=k
    if bool_legend:
        # ax.legend(loc='lower right', shadow=True)
        ax.legend(bbox_to_anchor=(1.08,0.5), borderaxespad=0., loc='center', shadow=True)
    ax.set_ylabel(ylabel, fontdict=font)
    # ax.set_xlim(0,35) # shared x
    # ax.set_ylim(0.85,1.45)

######################
# CSP
@dataclass
class CJHStylePlot():
    path2info_dot_dat: str = None # this is path to a file
    nrows: int          = 1
    ncols: int          = 1
    sharex: bool        = True
    sharey: bool        = False
    figsize: tuple      = (8, 9)
    facecolor: str      = 'w'
    edgecolor: str      = 'k'

    def __post_init__(self):

        if self.ncols != 1:
            self.sharex = False
            self.sharey = True

        if self.path2info_dot_dat is not None:
            """ This is an ACMSimC Plot"""
            # this is a directory 
            self.dir_of_info_dot_dat = os.path.dirname(os.path.abspath(self.path2info_dot_dat)) + '/'
            # print('path2info_dot_dat:', self.path2info_dot_dat)
            print('dir_of_info_dot_dat:', self.dir_of_info_dot_dat)

        else:
            """ This is general dataframe based plot """
            self.fig, self.axes_v = plt.subplots(  
                                         nrows=     self.nrows,
                                         ncols=     self.ncols,
                                         sharex=    self.sharex,
                                         sharey=    self.sharey,
                                         figsize=   self.figsize,
                                         facecolor= self.facecolor,
                                         edgecolor= self.edgecolor,
                                       ); # modifying dpi would change the size of fonts, so don't do that
            # fig.subplots_adjust(wspace=0.01)
            if self.nrows == 1:
                self.axes_v = [self.axes_v]
            if self.ncols != 1:
                self.axes_v = np.ravel(self.axes_v)

    """ ACMSimC Plot """
    def load(self):
        self.df_list, self.time, self.fname = load_dataframe(self.dir_of_info_dot_dat)
        return self.df_list, self.time, self.fname
    def plot(self, list_of_ylabel, list_of_signalNames, 
             customize_plot=lambda axes_v:None,
             customize_plot_after=lambda axes_v:None,
             index=0, tb=0, te=None, bool_sumatrapdf=True):
        if self.df_list is None:
            self.load()

        df, time, fname = self.df_list[index], self.time, self.fname

        b, e = 0, -1
        if tb != 0:
            CL_TS = time[1] - time[0]
            b = int(tb / CL_TS)
            if te is not None:
                e = int(te / CL_TS)

        ''' 图片对象在这里当场生成，使得外部可以轻松在调用csp.plot之前，通过修改csp.figsize来修改图片的大小。
        '''
        fig, axes_v = self.fig, self.axes_v = plt.subplots(  
                                 ncols=     self.ncols,
                                 nrows=     self.nrows,
                                 sharex=    self.sharex,
                                 figsize=   self.figsize,
                                 facecolor= self.facecolor,
                                 edgecolor= self.edgecolor,
                               ); # modifying dpi would change the size of fonts
        # fig.subplots_adjust(wspace=0.01)

        customize_plot(axes_v)

        for i, ylabel, signalNames in zip(range(self.nrows), list_of_ylabel, list_of_signalNames):
            ax = axes_v[i]
            OrDi = OD()
            for ind, signalName in enumerate(signalNames):
                OrDi[str(ind)] = df[signalName][b:e]
            plot_it(ax, time[b:e]-tb, ylabel, OrDi)

        for ax in axes_v:
            ax.grid(True)
            # ax.axvspan(14, 29, facecolor='r', alpha=0.1)
            ax.set_yticklabels([f'{el:g}' for el in ax.get_yticks()], font) # 为了修改ytick的字体  # https://matplotlib.org/3.3.3/api/_as_gen/matplotlib.axes.Axes.set_yticklabels.html

        # axes_v[-1].set_xlim((0,130))
        # axes_v[-1].set_xticks(np.arange(0, 201, 20))
        axes_v[-1].set_xlabel('Time, $t$ [s]', fontdict=font)
        axes_v[-1].set_xticklabels(axes_v[-1].get_xticks(), font)

        customize_plot_after(axes_v)

        # print(self.path2info_dot_dat)
        # print(fname[:-4])
        # print(fname)
        # fig.tight_layout() # tight layout is bad if you have too many subplots. Use bbox_inches instead
        path2outputFile = self.dir_of_info_dot_dat+fname[:-4] + f'-{index+1:03d}'
        fig.savefig(              f'{path2outputFile}.pdf', dpi=300, bbox_inches='tight', pad_inches=0)
        if bool_sumatrapdf:
            os.system('sumatrapdf ' + f'{path2outputFile}.pdf')
        os.system(f'pdfcrop         {path2outputFile}.pdf \
                                    {path2outputFile}-crop.pdf')
        # plt.show()

    """ General Plot for Academic Papers """
    def aca_plot(self, dict_label_dataframe, # a dict of ylabel: df
                    x='Time', # df[x] is usually time
                    y='[V]',  # df[y] is usually signal
                    b=0, e=-1, # truncate data (plot only the part we are interested in)
                    begin=0, end=None, # shift in time (x-axis)
                    adjust_figure=lambda axes_v:None, # additioanl ax properties manipulation before plot
                    adjust_ylabel=lambda axes_v:None, # additioanl ax properties manipulation after plot
                ):

        """ Must adjust this before set_yticklabels or else the ticks could be wrong """
        adjust_figure(self.axes_v)
        for ylabel, ax in zip(dict_label_dataframe.keys(), self.axes_v):
            for df, style in zip(dict_label_dataframe[ylabel], ['k-', 'r--', 'b--', 'g--']):
                alpha = 1.0
                if len(dict_label_dataframe[ylabel] ) == 2:
                    # If there are two traces, the black one is dashed and red is solid.
                    if style[0] == 'k':
                        style = 'k--'
                    if style[0] == 'r':
                        style = 'r-'
                        alpha = 0.7
                ax.plot(df[x][b:e]-df[x].values[0]-begin, df[y][b:e], style, alpha=alpha, lw=0.8)
            ax.set_ylabel(ylabel, fontdict=font, multialignment='center')
            # print('\t', ylabel)
            ax.set_yticklabels([f'{el:g}' for el in ax.get_yticks()], fontdict=font)

        for ax in self.axes_v:
            ax.grid(True)
            # ax.axvspan(14, 29, facecolor='r', alpha=0.1)
            # ax.set_yticklabels([f'{el:g}' for el in ax.get_yticks()], font) # 为了修改ytick的字体  # https://matplotlib.org/3.3.3/api/_as_gen/matplotlib.axes.Axes.set_yticklabels.html

            if self.ncols != 1:
                ax.set_xlabel('Time, $t$ [s]', fontdict=font)
                ax.set_xticklabels([f'{el:g}' for el in self.axes_v[-1].get_xticks()], fontdict=font)

        if end is not None:
            print('\tTime range:', [0, end-begin])
            self.axes_v[-1].set_xlim([0, end-begin])

        if self.ncols == 1:
            self.axes_v[-1].set_xlabel('Time, $t$ [s]', fontdict=font)
            self.axes_v[-1].set_xticklabels([f'{el:g}' for el in self.axes_v[-1].get_xticks()], fontdict=font)

        ''' Move this to the end '''
        adjust_ylabel(self.axes_v)

        return self.axes_v
    def aca_save(self, path2outputFile, bool_sumatrapdf=True, fig=None):
        if fig is None:
            fig = self.fig
        fig.savefig(              f'{path2outputFile}.pdf', dpi=400, bbox_inches='tight', pad_inches=0)
        os.system(f'pdfcrop         "{path2outputFile}.pdf" \
                                    "{path2outputFile}-crop.pdf"')
        if bool_sumatrapdf:
            os.system('sumatrapdf ' + f'"{path2outputFile}-crop.pdf"')

if __name__ == '__main__':
    # import emachinery
    # CJHStylePlot = emachinery.acmsimcv5.CJHAcademicPlot.__cjhAcademicPlotSettings.CJHStylePlot
    csp = CJHStylePlot( path2info_dot_dat=r'D:\DrH\Codes\emachineryTestPYPI\emachinery\acmsimcv5\dat\IM_Marino05_PI-Ohtani/info.dat', 
                        nrows=6)

    list_of_ylabel = [
      'Speed\n[rpm]',
      '$\\alpha$-axis Flux\n[Wb]',
      'Torque Current\n[A]',
      'Offset Voltage\n[V]',
      'Magn. Current\n[A]',
      'Load Torque\n[Nm]',
    ]

    list_of_signalNames = [
    ######################################
    ("ACM.rpm_cmd",
    "ACM.rpm",
    "marino.xOmg*ELEC_RAD_PER_SEC_2_RPM",
    ),
    ######################################
    ("ACM.psi_Amu",
    "ohtani.psi_2_real_output[0]",
    ),
    ######################################
    ("CTRL.I->iDQ_cmd[1]",
    # "ACM.iTs",
    "CTRL.I->iDQ[1]",
    ) ,
    ######################################
    ("ohtani.correction_integral_term[0]",
    "ohtani.correction_integral_term[1]",
    ) ,
    ######################################
    ("CTRL.I->iDQ_cmd[0]",
    # "ACM.iMs",
    "CTRL.I->iDQ[0]",
    ) ,
    ######################################
    ("ACM.TLoad",
    "marino.xTL",
    ),
    ]

    def customize_plot(axes_v):
        axes_v[0].set_ylim([-300, 300])
    csp.plot(list_of_ylabel, list_of_signalNames, customize_plot=customize_plot)

if __name__ == '__main__':
    # %%
    # import sys
    # sys.path.insert(1, r'D:\DrH\Codes\emachineryTestPYPI\emachinery\acmsimcv5\CJHAcademicPlot')
    from __cjhAcademicPlotSettings import CJHStylePlot, pd, np, plt, os

    ''' Current Function
    '''
    I_AMPL = 1.0
    PERIOD = 0.02 # s
    CL_TS = 1e-4
    time = np.arange(0, PERIOD, CL_TS)
    phase_angle = np.arange(-np.pi, np.pi, 2*np.pi / len(time))

    """ Compensation Voltage Function By Current Value """
    def sigmoid(x, a1=0, a2=2, a3=10):
        return a1 * x + a2 / (1.0 + np.exp(-a3 * x)) - a2*0.5

    def main():
        dict_label_dataframe = dict()

        for key, I_AMPL in zip(['1', '2'], [1, 0.2]):
            phase_current = np.sin(phase_angle) * I_AMPL
            ui_curve = [sigmoid(current_value) for current_value in phase_current]
            dict_label_dataframe.update(
                {
                    key: [
                        pd.DataFrame({
                            'Time': time,
                            'Signal': phase_current
                        }), 
                        pd.DataFrame({
                            'Time': time,
                            'Signal': ui_curve
                        }), 
                    ] # 如果不是一个list，则CSP会报错：TypeError: string indices must be integers
                }
            )

        def adjust_ylabel(axes_v):
            for ax in axes_v:
                ax.set_ylabel('[A] or [V]')

        csp = CJHStylePlot( nrows=2, figsize=(4, 3) )
        csp.aca_plot(dict_label_dataframe, x='Time', y='Signal', 
                        adjust_ylabel=adjust_ylabel)
        csp.axes_v[0].set_ylim([-1.1,1.1])
        csp.aca_save(f'{os.path.dirname(__file__)}/compensation_votlage_shape', bool_sumatrapdf=True)

    main()


#%%
from pylab import plt, np, mpl
plt.figure(99); plt.plot([1,2,3]) # for sutpid sublime 4
# from scipy._lib.six import u
plt.style.use('classic')
mpl.rcParams['mathtext.fontset'] = 'stix'
mpl.rcParams['font.family'] = 'STIXGeneral' # ['serif'] # ['Times New Roman']
mpl.rcParams['legend.fontsize'] = 13
mpl.rcParams['font.size'] = 16 # font size for xticks, yticks
fontdict = {'family' : 'Times New Roman', #'serif',
        'color' : 'darkblue',
        'weight' : 'normal',
        'size' : 16,}
import pandas as pd
import os
from scipy.optimize import curve_fit
# GLOBAL_R = 2.48785


def sigmoid_ori(x, a3):
    return 1 / (1+np.exp(-a3 * x))
def linear(x, a1, a2):
    return a1 * x + a2
def wt_sigmoid(x, a1, a2, a3):
    return a1 * x + a2 / (1+np.exp(-a3 * x)) - a2/2
def zyf_sigmoid(x, a1, a2, a3, a4, a5):
    return a1 * x + a2 / (1+np.exp(-a3 * x)) - a2/2 \
                  + a4 / (1+np.exp(-a5 * x)) - a4/2
def three_sigmoid(x, a1, a2, a3, a4, a5, a6, a7):
    return a1 * x + a2 / (1+np.exp(-a3 * x)) - a2/2 \
                  + a4 / (1+np.exp(-a5 * x)) - a4/2 \
                  + a6 / (1+np.exp(-a7 * x)) - a6/2
def fit_atan(x, a1, a2, a3):
    return a1*x + a2*np.arctan(a3*x)
def fit_tanh(x, a1, a2, a3):
    return a1*x + a2*np.tanh(a3*x)
def fit_atan_plus_tanh(x, a1, a2, a3, a4, a5):
    return a1*x + a2*np.arctan(a3*x) + a4*np.tanh(a5*x)
def fit_sigmoid_plus_atan(x, a1, a2, a3, a4, a5):
    return a1*x + a2 / (1+np.exp(-a3 * x)) - a2/2 + a4*np.arctan(a5*x)
def fit_sigmoid_plus_tanh(x, a1, a2, a3, a4, a5):
    return a1*x + a2 / (1+np.exp(-a3 * x)) + a4*np.tanh(a5*x)

def analyze_ui_curve(i_data, u_data, _):
    ''' 拨乱反正，保证电流是从 -3, -1, 0, 1, 3 A 的顺序，电压同样调整顺序
    '''
    # 注意sort方法是直接修改对象本身的，所以要先改电压，再改电流
    # u_data.sort(key=dict(zip(u_data, i_data)).get) # old method
    # u_data = [el for _, el in sorted(zip(i_data, u_data), key=lambda pair: pair[0])] # new method
    u_data = [el for _, el in sorted(zip(i_data, u_data))] # new method shortest
    i_data.sort()
    # 倒序，负电流在前。
    x = i_data = i_data[::-1]
    y = u_data = u_data[::-1]
    xdata, ydata = np.asarray(x), np.asarray(y)
    plot_xdata = np.arange(min(xdata)*3, max(xdata)*3, 0.001)


    ''' Prepare figure for publication 
    '''

    # Figure 10
    plt.figure(10, figsize=(5,4))
    plt.title('U-I Curves')

    plt.grid(1)
    plt.plot(xdata, ydata, 'ko', label='original data', alpha=0.3)
    # Get fitting_func
    for index, fitting_func in enumerate([  wt_sigmoid, 
                                            #zyf_sigmoid, 
                                            #three_sigmoid, 
                                            #fit_atan, 
                                            #fit_tanh, 
                                            #fit_atan_plus_tanh, 
                                            #fit_sigmoid_plus_atan,
                                            #fit_sigmoid_plus_tanh,
                                        ]):
        ''' curve fitting
        '''
        try:
            popt, pcov = curve_fit(fitting_func, xdata, ydata)
            perr = np.sqrt(np.diag(pcov))
        except RuntimeError as e:
            print(e)
            continue

        print('index =', index)
        print('popt =', popt)
        # print('pcov =', pcov)
        # print('perr =', perr)
        print('\tSquare Error Sum:', sum( (fitting_func(xdata, *popt) - ydata)**2 ))
        print('\tAbs Error Sum:', sum( abs(fitting_func(xdata, *popt) - ydata) ))
        print()

        plt.plot(plot_xdata, fitting_func(plot_xdata, *popt),          'r--', alpha=0.8, label=str(index))
        plt.plot(plot_xdata, fitting_func(plot_xdata, 0.0, *popt[1:]), 'b-.', alpha=0.8, label=str(index), lw=2)

        # plt.plot(plot_xdata, wt_sigmoid(plot_xdata, *popt),                 'r--', alpha=0.8, label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f' % tuple(popt))
        # plt.plot(plot_xdata, wt_sigmoid(plot_xdata, 0.0, popt[1], popt[2]), 'b-.', alpha=0.8, label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f' % tuple(popt), lw=2)
        # plt.plot(plot_xdata, zyf_sigmoid(plot_xdata, *popt),          'r--', alpha=0.8, label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f, a4=%5.3f, a5=%5.3f' % tuple(popt))
        # plt.plot(plot_xdata, zyf_sigmoid(plot_xdata, 0.0, *popt[1:]), 'b-.', alpha=0.8, label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f, a4=%5.3f, a5=%5.3f' % tuple(popt), lw=2)
        # plt.plot(plot_xdata, three_sigmoid(plot_xdata, *popt),          'r--', alpha=0.8, label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f, a4=%5.3f, a5=%5.3f, a6=%5.3f, a7=%5.3f' % tuple(popt))
        # plt.plot(plot_xdata, three_sigmoid(plot_xdata, 0.0, *popt[1:]), 'b-.', alpha=0.8, label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f, a4=%5.3f, a5=%5.3f, a6=%5.3f, a7=%5.3f' % tuple(popt), lw=2)
        # plt.show()

    if True:
        plt.grid(1)
        plt.gca().lines[0].set_label('Measured data')
        plt.gca().lines[1].set_label('Fitted Sigmoid curve')
        plt.gca().lines[2].set_label('Fitted Sigmoid curve wo/ linear term')
        # plt.legend(loc='best')
        # plt.xlim([-5,5.1])
        # plt.ylim([-15,15.1])
        plt.xlabel('Phase $b$ current, $i_b$ [A]')
        plt.ylabel('Phase $b$ voltage, $u_{bn}$ [V]')

        # Note  D_b =  u_{bn,\\rm dist}
        # plt.annotate('$D_b$', xy=(3,6.5), xytext=(3,1), color='blue', size=24,
        #                 arrowprops = dict(facecolor ='blue', width=2),)

        plt.xlim([0,5])
        plt.ylim([0,25])
        plt.savefig(os.path.dirname(__file__)+f'/UIC-SlessInvPaper.pdf', dpi=400, bbox_inches = "tight")
        # fname = os.path.split(tag_selected)[-1]
        # plt.savefig(os.path.dirname(__file__)+f'/{fname}.pdf', dpi=400, bbox_inches = "tight")
        # plt.savefig(fr'D:\DrH\[00]GetWorking\200 The-ACM-Book\ACMBook_TEX\images\inverter-nonl-fitting-@{tag_selected}.pdf', dpi=400, bbox_inches = "tight")


    # Sub-figure for Slessinv.R1
    plt.figure(100+_, figsize=(5/2,4/2))
    if _ == 0:
        plt.title(r'INV2, $V_{dc}=150$ V')
    elif _ == 1:
        plt.title(r'INV2, $V_{dc}=300$ V')
    elif _ == 2:
        plt.title(r'INV1, $V_{dc}=150$ V')
    elif _ == 3:
        plt.title(r'INV1, $V_{dc}=300$ V')
    plt.grid(1)
    plt.plot(xdata, ydata, 'ko', label='original data', alpha=0.3)
    plt.plot(plot_xdata, fitting_func(plot_xdata, *popt),          'r--', alpha=0.8, label=str(index))
    plt.plot(plot_xdata, fitting_func(plot_xdata, 0.0, *popt[1:]), 'b-.', alpha=0.8, label=str(index), lw=2)
    plt.grid(1)
    plt.gca().lines[0].set_label('Measured data')
    plt.gca().lines[1].set_label('Fitted Sigmoid curve')
    plt.gca().lines[2].set_label('Fitted Sigmoid curve wo/ linear term')
    plt.xlabel('$i_b$ [A]')
    plt.ylabel('$u_{bn}$ [V]')
    plt.xlim([0,5])
    plt.ylim([0,25])
    plt.savefig(os.path.dirname(__file__)+f'/UIC-SlessInvPaper-{_}.pdf', dpi=400, bbox_inches = "tight")




    ''' draw the partial derivative w.r.t. a3
    '''
    if False:
        a2, a3 = popt[1], popt[2]
        plt.plot(x, a2*x*sigmoid_ori(x, a3)*(1-sigmoid_ori(x, a3)), 'k-.', label='derivative')

        popt_ori = popt[2]
        ''' 观察sigmoid的变化当a3变化
        '''
        # popt[2] = popt_ori * 1.2
        # plt.plot(x, wt_sigmoid(x, 0.0, popt[1], popt[2]), 'b-.', color='black', alpha=alpha,
        #      label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f' % tuple(popt), lw=2)
        # popt[2] = popt_ori * 0.8
        # plt.plot(x, wt_sigmoid(x, 0.0, popt[1], popt[2]), 'b-.', color='green', alpha=alpha,
        #      label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f' % tuple(popt), lw=2)
        popt[2] = popt_ori * 0.5
        plt.plot(x, wt_sigmoid(x, 0.0, popt[1], popt[2]), 'b-.', color='black', alpha=alpha,
                label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f' % tuple(popt), lw=2)
        popt[2] = popt_ori * 100
        plt.plot(x, wt_sigmoid(x, 0.0, popt[1], popt[2]), 'b-.', color='green', alpha=alpha,
                label='fit: a1=%5.3f, a2=%5.3f, a3=%5.3f' % tuple(popt), lw=2)
        popt[2] = popt_ori

def is_float(text):
    # by mathfac, https://stackoverflow.com/questions/354038/how-do-i-check-if-a-string-is-a-number-float
    try:
        float(text)
        # check for nan/infinity etc.
        if text.isalpha():
            return False
        return True
    except ValueError:
        return False


''' 决定用哪个数据
# 从字符串里提取出电流、电压数据
'''
def synthesize_ccs_memory_phaseCurrent(tag_selected):

    df_voltage = pd.read_csv(os.path.dirname(__file__)+'./逆变器非线性测试数据（保存内存）/'+tag_selected+'_Voltage.dat', skiprows=0)
    df_current = pd.read_csv(os.path.dirname(__file__)+'./逆变器非线性测试数据（保存内存）/'+tag_selected+'_Current.dat', skiprows=0)
    # print(df_voltage)
    # print(df_current)
    flatten = lambda t: [item for sublist in t for item in sublist]
    return flatten(df_current.values), flatten(df_voltage.values)




if __name__ == '__main__':
   
    ''' Select a tag
    '''
    # tag_selected = '20210622/OW_IPM_Inverter_80V_NoDeadTimeCompensation' # max I = 1.0 A
    # tag_selected = '20210622/OW_IPM_Inverter_180V_NoDeadTimeCompensation' # max I = 1.0 A
    # tag_selected = '20210623/OW_IPM_Inverter_80V_NoDeadTimeCompensation' # max I = 4.2 A
    # tag_selected = '20210623/OW_IPM_Inverter_180V_NoDeadTimeCompensation' # max I = 4.2 A

    # tag_selected = '20210725/SiC_Inverter_150V_NoDeadTimeCompensation'
    # tag_selected = '20210725/SiC_Inverter_300V_NoDeadTimeCompensation'

    # tag_selected = '20210725/OW-IPM_Inverter_100V_NoDeadTimeCompensation'
    # tag_selected = '20210725/OW-IPM_Inverter_150V_NoDeadTimeCompensation'
    # tag_selected = '20210725/OW-IPM_Inverter_300V_NoDeadTimeCompensation'

    ''' This is used for SlessInv paper.
    '''
    for _, tag_selected in enumerate(
                        ['20210725/SiC_Inverter_150V_NoDeadTimeCompensation',
                         '20210725/SiC_Inverter_300V_NoDeadTimeCompensation',
                         # '20210725/OW-IPM_Inverter_100V_NoDeadTimeCompensation',
                         '20210725/OW-IPM_Inverter_150V_NoDeadTimeCompensation',
                         '20210725/OW-IPM_Inverter_300V_NoDeadTimeCompensation']):

    # if False:

        phase_current_data, phase_voltage_data = dict(), dict()
        phase_current_data[tag_selected], \
        phase_voltage_data[tag_selected] = i_data, u_data = synthesize_ccs_memory_phaseCurrent(tag_selected)
        print('len(current) =', len(i_data))
        print('len(voltage) =', len(u_data))

        ''' LUT
        '''
        if True:
            ''' 生成二段式查表
            '''
            last_I = None
            I_incremental = None
            low_current_I_incremental_list = []
            low_current_voltage_list = []
            low_current_current_list = []
            high_current_I_incremental_list = []
            high_current_voltage_list = []
            high_current_current_list = []
            for index, ui_pair in enumerate(zip(u_data, i_data)):
                U, I = ui_pair
                if I <= 0:
                    continue
                # 只用正电流数据
                if last_I is not None:
                    if I_incremental is None:
                        I_BigStep = last_I - I
        
                    I_incremental = last_I - I
                    # print(f'{[index]} increment in I: {I_incremental:g} A')

                    if abs(I_incremental - I_BigStep) < 0.1*I_BigStep:
                        high_current_current_list.append(I)
                        high_current_voltage_list.append(U)
                        high_current_I_incremental_list.append(I_incremental)
                    else:
                        low_current_current_list.append(I)
                        low_current_voltage_list.append(U)
                        low_current_I_incremental_list.append(I_incremental)
                else:
                    print('Max I:', I)
                last_I = I
            average = lambda ell: sum(ell)/len(ell)
            # print(high_current_voltage_list)
            I_BigStep = average(high_current_I_incremental_list)
            # print(I_BigStep, 'A')
            # print(low_current_voltage_list)
            I_SmallStep = average(low_current_I_incremental_list)
            # print(I_SmallStep, 'A')

            ''' Fit high current data to a line 
            '''
            low_current_voltage_list  = np.asarray(low_current_voltage_list  )
            low_current_current_list  = np.asarray(low_current_current_list  )
            high_current_voltage_list = np.asarray(high_current_voltage_list )
            high_current_current_list = np.asarray(high_current_current_list )
            # print(low_current_current_list)
            # print(high_current_current_list)
            # quit()
            xdata = high_current_current_list[:10] # 前面10个点的电流、电压是最大的哦
            ydata = high_current_voltage_list[:10] # 前面10个点的电流、电压是最大的哦
            popt, pcov = curve_fit(linear, xdata, ydata)
            Fitted_R = popt[0]
            print('Fitting for Resistance')
            print('\tpopt =', popt)
            print('\tSquare Error Sum:', sum(    (linear(xdata, *popt) - ydata)**2 ))
            print('\tAbs Error Sum:',    sum( abs(linear(xdata, *popt) - ydata)    ))
            print('\t此处，Fitted_R 必须用大电流的结果来替代！！！')

            ''' The LUT Method Generator
            '''
            print('\t#define FITTED_R', Fitted_R)
            print('\t#define LUT_STEPSIZE_BIG ', I_BigStep)
            print('\t#define LUT_STEPSIZE_SMALL ', I_SmallStep)
            print('\t#define LUT_STEPSIZE_BIG_INVERSE ', 1.0/I_BigStep)
            print('\t#define LUT_STEPSIZE_SMALL_INVERSE ', 1.0/I_SmallStep)
            lut_lc_voltage = low_current_voltage_list - Fitted_R*low_current_current_list
            lut_hc_voltage = high_current_voltage_list - Fitted_R*high_current_current_list
            lut_lc_voltage = [0] + list(lut_lc_voltage[::-1])
            lut_hc_voltage = list(lut_hc_voltage[::-1])
            N_LC = len(lut_lc_voltage); print('Low current data points:', N_LC)
            N_HC = len(lut_hc_voltage); print('High current data points:', N_HC)
            print('\t#define LUT_N_LC ', N_LC)
            print('\t#define LUT_N_HC ', N_HC)
            print(f'\tREAL lut_lc_voltage[{N_LC}] = {{' + ', '.join(f'{el:g}' for el in lut_lc_voltage) + '};')
            print(f'\tREAL lut_hc_voltage[{N_HC}] = {{' + ', '.join(f'{el:g}' for el in lut_hc_voltage) + '};')

            plt.figure(21)
            plt.plot(xdata, ydata,                         'ko', alpha=0.8, label='Ori')
            plt.plot(high_current_current_list, high_current_voltage_list - Fitted_R*high_current_current_list, 'rx', alpha=0.8, label='Ori')
            plt.plot(low_current_current_list, low_current_voltage_list - Fitted_R*low_current_current_list, 'b^', alpha=0.8, label='Ori')
            plt.plot(xdata, linear(xdata, *popt),          'r--', alpha=0.8, label='Fit')
            plt.plot(xdata, linear(xdata, 0.0, *popt[1:]), 'b-.', alpha=0.8, label='Plateau', lw=2)

            LUT_N_LC = N_LC
            LUT_N_HC = N_HC
            LUT_STEPSIZE_BIG = I_BigStep
            LUT_STEPSIZE_SMALL = I_SmallStep
            LUT_STEPSIZE_BIG_INVERSE = 1.0/I_BigStep
            LUT_STEPSIZE_SMALL_INVERSE = 1.0/I_SmallStep
            LUT_I_TURNING_LC = LUT_N_LC*LUT_STEPSIZE_SMALL
            LUT_I_TURNING_HC = LUT_N_LC*LUT_STEPSIZE_SMALL + LUT_N_HC*LUT_STEPSIZE_BIG
            V_PLATEAU = lut_hc_voltage[-1]
            print('\t#define LUT_STEPSIZE_BIG',             I_BigStep)
            print('\t#define LUT_STEPSIZE_SMALL',           I_SmallStep)
            print('\t#define LUT_STEPSIZE_BIG_INVERSE',     1.0/I_BigStep)
            print('\t#define LUT_STEPSIZE_SMALL_INVERSE',   1.0/I_SmallStep)
            print('\t#define LUT_I_TURNING_LC',             LUT_N_LC*LUT_STEPSIZE_SMALL)
            print('\t#define LUT_I_TURNING_HC',             LUT_N_LC*LUT_STEPSIZE_SMALL + LUT_N_HC*LUT_STEPSIZE_BIG)
            print('\t#define V_PLATEAU',                    lut_hc_voltage[-1])
            def lookup_compensation_voltage_by_index(current_value):
                abs_current_value = abs(current_value)

                if abs_current_value < LUT_I_TURNING_LC:
                    float_index = abs_current_value * LUT_STEPSIZE_SMALL_INVERSE
                    index = int(float_index)
                    if index+1 >= LUT_N_LC:
                        # print('DEBUG', index)
                        slope = (lut_hc_voltage[0] - lut_lc_voltage[index]) * LUT_STEPSIZE_SMALL_INVERSE
                            # 经过下面的电流列表验证，这里确实应该是用LUT_STEPSIZE_SMALL_INVERSE
                            # print(low_current_current_list)
                            # print(high_current_current_list)
                            # quit()
                        # print('Turning voltages:', lut_hc_voltage[0], lut_lc_voltage[index])
                        # print(index, 'Low--')
                    else:
                        slope = (lut_lc_voltage[index+1] - lut_lc_voltage[index]) * LUT_STEPSIZE_SMALL_INVERSE
                        # print(index, 'Low')
                    return np.sign(current_value) * (lut_lc_voltage[index] + slope * (abs_current_value - index*LUT_STEPSIZE_SMALL))
                # elif abs_current_value < LUT_I_TURNING_HC:
                else:
                    float_index = (abs_current_value - LUT_I_TURNING_LC) * LUT_STEPSIZE_BIG_INVERSE
                    index = int(float_index)
                    if index+1 >= LUT_N_HC:
                        # print('Plateau')
                        return V_PLATEAU
                    else:
                        # print(index, 'High')
                        slope = (lut_hc_voltage[index+1] - lut_hc_voltage[index]) * LUT_STEPSIZE_BIG_INVERSE
                    return np.sign(current_value) * (lut_hc_voltage[index] + slope * (abs_current_value - LUT_I_TURNING_LC - index*LUT_STEPSIZE_BIG))

            plot_xdata = np.arange(0, max(xdata)+0.1, 1e-3)
            # plot_xdata = np.arange(0, max(xdata)+0.1, 0.5*LUT_STEPSIZE_SMALL)
            plt.figure(22)
            plt.title('LUT')
            plt.plot(plot_xdata, [lookup_compensation_voltage_by_index(el) for el in plot_xdata], 'k.', alpha=0.8, label='LUT')
            plt.plot(i_data, np.array(u_data)-Fitted_R*np.array(i_data), 'o', alpha=0.8, label='Ori')
            plt.grid()
            plt.legend()
            print('结论，电阻值辨识得用足够大的电流才靠谱！')
            # plt.show()

        ''' 描点、拟合、画图
        '''
        analyze_ui_curve(i_data, u_data, _)

    print('Note a2 in this script is inconsistent with the SlessInv paper.')
    plt.show()

# %%



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_pages.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from mplwidget import MplWidget


class Ui_MainPages(object):
    def setupUi(self, MainPages):
        if not MainPages.objectName():
            MainPages.setObjectName(u"MainPages")
        MainPages.resize(1476, 1028)
        self.main_pages_layout = QVBoxLayout(MainPages)
        self.main_pages_layout.setSpacing(0)
        self.main_pages_layout.setObjectName(u"main_pages_layout")
        self.main_pages_layout.setContentsMargins(5, 5, 5, 5)
        self.pages = QStackedWidget(MainPages)
        self.pages.setObjectName(u"pages")
        self.page_1 = QWidget()
        self.page_1.setObjectName(u"page_1")
        self.page_1.setStyleSheet(u"font-size: 14pt")
        self.page_1_layout = QVBoxLayout(self.page_1)
        self.page_1_layout.setSpacing(5)
        self.page_1_layout.setObjectName(u"page_1_layout")
        self.page_1_layout.setContentsMargins(5, 5, 5, 5)
        self.welcome_base = QFrame(self.page_1)
        self.welcome_base.setObjectName(u"welcome_base")
        self.welcome_base.setMinimumSize(QSize(300, 150))
        self.welcome_base.setMaximumSize(QSize(300, 150))
        self.welcome_base.setFrameShape(QFrame.NoFrame)
        self.welcome_base.setFrameShadow(QFrame.Raised)
        self.center_page_layout = QVBoxLayout(self.welcome_base)
        self.center_page_layout.setSpacing(10)
        self.center_page_layout.setObjectName(u"center_page_layout")
        self.center_page_layout.setContentsMargins(0, 0, 0, 0)
        self.logo = QFrame(self.welcome_base)
        self.logo.setObjectName(u"logo")
        self.logo.setMinimumSize(QSize(300, 120))
        self.logo.setMaximumSize(QSize(300, 120))
        self.logo.setFrameShape(QFrame.NoFrame)
        self.logo.setFrameShadow(QFrame.Raised)
        self.logo_layout = QVBoxLayout(self.logo)
        self.logo_layout.setSpacing(0)
        self.logo_layout.setObjectName(u"logo_layout")
        self.logo_layout.setContentsMargins(0, 0, 0, 0)

        self.center_page_layout.addWidget(self.logo)

        self.label = QLabel(self.welcome_base)
        self.label.setObjectName(u"label")
        self.label.setAlignment(Qt.AlignCenter)

        self.center_page_layout.addWidget(self.label)


        self.page_1_layout.addWidget(self.welcome_base, 0, Qt.AlignHCenter)

        self.pages.addWidget(self.page_1)
        self.page_2 = QWidget()
        self.page_2.setObjectName(u"page_2")
        self.page_2_layout = QVBoxLayout(self.page_2)
        self.page_2_layout.setSpacing(5)
        self.page_2_layout.setObjectName(u"page_2_layout")
        self.page_2_layout.setContentsMargins(5, 5, 5, 5)
        self.scroll_area = QScrollArea(self.page_2)
        self.scroll_area.setObjectName(u"scroll_area")
        self.scroll_area.setStyleSheet(u"background: transparent;")
        self.scroll_area.setFrameShape(QFrame.NoFrame)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setWidgetResizable(True)
        self.contents = QWidget()
        self.contents.setObjectName(u"contents")
        self.contents.setGeometry(QRect(0, 0, 219, 255))
        self.contents.setStyleSheet(u"background: transparent;")
        self.verticalLayout = QVBoxLayout(self.contents)
        self.verticalLayout.setSpacing(15)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(5, 5, 5, 5)
        self.title_label = QLabel(self.contents)
        self.title_label.setObjectName(u"title_label")
        self.title_label.setMaximumSize(QSize(16777215, 40))
        font = QFont()
        font.setPointSize(16)
        self.title_label.setFont(font)
        self.title_label.setStyleSheet(u"font-size: 16pt")
        self.title_label.setAlignment(Qt.AlignCenter)

        self.verticalLayout.addWidget(self.title_label)

        self.description_label = QLabel(self.contents)
        self.description_label.setObjectName(u"description_label")
        self.description_label.setAlignment(Qt.AlignHCenter|Qt.AlignTop)
        self.description_label.setWordWrap(True)

        self.verticalLayout.addWidget(self.description_label)

        self.row_1_layout = QHBoxLayout()
        self.row_1_layout.setObjectName(u"row_1_layout")

        self.verticalLayout.addLayout(self.row_1_layout)

        self.row_2_layout = QHBoxLayout()
        self.row_2_layout.setObjectName(u"row_2_layout")

        self.verticalLayout.addLayout(self.row_2_layout)

        self.row_3_layout = QHBoxLayout()
        self.row_3_layout.setObjectName(u"row_3_layout")

        self.verticalLayout.addLayout(self.row_3_layout)

        self.row_4_layout = QVBoxLayout()
        self.row_4_layout.setObjectName(u"row_4_layout")

        self.verticalLayout.addLayout(self.row_4_layout)

        self.row_5_layout = QVBoxLayout()
        self.row_5_layout.setObjectName(u"row_5_layout")

        self.verticalLayout.addLayout(self.row_5_layout)

        self.scroll_area.setWidget(self.contents)

        self.page_2_layout.addWidget(self.scroll_area)

        self.pages.addWidget(self.page_2)
        self.page_3 = QWidget()
        self.page_3.setObjectName(u"page_3")
        self.page_3.setStyleSheet(u"QFrame {\n"
"	font-size: 16pt;\n"
"}")
        self.page_3_layout = QVBoxLayout(self.page_3)
        self.page_3_layout.setObjectName(u"page_3_layout")
        self.plainTextEdit_NumbaScopeDict = QPlainTextEdit(self.page_3)
        self.plainTextEdit_NumbaScopeDict.setObjectName(u"plainTextEdit_NumbaScopeDict")

        self.page_3_layout.addWidget(self.plainTextEdit_NumbaScopeDict)

        self.pages.addWidget(self.page_3)
        self.page_4_scope = QWidget()
        self.page_4_scope.setObjectName(u"page_4_scope")
        self.horizontalLayout = QHBoxLayout(self.page_4_scope)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.scrollArea_4_scope = QScrollArea(self.page_4_scope)
        self.scrollArea_4_scope.setObjectName(u"scrollArea_4_scope")
        self.scrollArea_4_scope.setMinimumSize(QSize(0, 800))
        self.scrollArea_4_scope.setFrameShape(QFrame.NoFrame)
        self.scrollArea_4_scope.setFrameShadow(QFrame.Plain)
        self.scrollArea_4_scope.setWidgetResizable(True)
        self.scrollAreaWidgetContents_4_scope = QWidget()
        self.scrollAreaWidgetContents_4_scope.setObjectName(u"scrollAreaWidgetContents_4_scope")
        self.scrollAreaWidgetContents_4_scope.setGeometry(QRect(0, 0, 1448, 1000))
        self.verticalLayout_16 = QVBoxLayout(self.scrollAreaWidgetContents_4_scope)
        self.verticalLayout_16.setSpacing(5)
        self.verticalLayout_16.setObjectName(u"verticalLayout_16")
        self.verticalLayout_16.setContentsMargins(0, 0, 0, 0)
        self.MplWidget_ACMPlot = MplWidget(self.scrollAreaWidgetContents_4_scope)
        self.MplWidget_ACMPlot.setObjectName(u"MplWidget_ACMPlot")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.MplWidget_ACMPlot.sizePolicy().hasHeightForWidth())
        self.MplWidget_ACMPlot.setSizePolicy(sizePolicy)
        self.MplWidget_ACMPlot.setMinimumSize(QSize(800, 800))
        self.MplWidget_ACMPlot.setStyleSheet(u"background-color: rgb(170, 0, 127)")

        self.verticalLayout_16.addWidget(self.MplWidget_ACMPlot)

        self.scrollArea_4_scope.setWidget(self.scrollAreaWidgetContents_4_scope)

        self.horizontalLayout.addWidget(self.scrollArea_4_scope)

        self.pages.addWidget(self.page_4_scope)

        self.main_pages_layout.addWidget(self.pages)


        self.retranslateUi(MainPages)

        self.pages.setCurrentIndex(2)


        QMetaObject.connectSlotsByName(MainPages)
    # setupUi

    def retranslateUi(self, MainPages):
        MainPages.setWindowTitle(QCoreApplication.translate("MainPages", u"Form", None))
        self.label.setText(QCoreApplication.translate("MainPages", u"Welcome To PyOneDark GUI", None))
        self.title_label.setText(QCoreApplication.translate("MainPages", u"Custom Widgets Page", None))
        self.description_label.setText(QCoreApplication.translate("MainPages", u"Here will be all the custom widgets, they will be added over time on this page.\n"
"I will try to always record a new tutorial when adding a new Widget and updating the project on Patreon before launching on GitHub and GitHub after the public release.", None))
    # retranslateUi




# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# IMPORT BUTTON AND DIV
# ///////////////////////////////////////////////////////////////
from . py_left_menu_button import PyLeftMenuButton
from . py_div import PyDiv

# IMPORT FUNCTIONS
# ///////////////////////////////////////////////////////////////
from gui.core.functions import *

# PY LEFT MENU
# ///////////////////////////////////////////////////////////////
class PyLeftMenu(QWidget):
    # SIGNALS
    clicked = Signal(object)
    released = Signal(object)

    def __init__(
        self,
        parent = None,
        app_parent = None,
        dark_one = "#1b1e23",
        dark_three = "#21252d",
        dark_four = "#272c36",
        bg_one = "#2c313c",
        icon_color = "#c3ccdf",
        icon_color_hover = "#dce1ec",
        icon_color_pressed = "#edf0f5",
        icon_color_active = "#f5f6f9",
        context_color = "#568af2",
        text_foreground = "#8a95aa",
        text_active = "#dce1ec",
        duration_time = 500,
        radius = 8,
        minimum_width = 50,
        maximum_width = 240,
        icon_path = "icon_menu.svg",
        icon_path_close = "icon_menu_close.svg",
        toggle_text = "Hide Menu",
        toggle_tooltip = "Show menu"
    ):
        super().__init__()

        # PROPERTIES
        # ///////////////////////////////////////////////////////////////
        self._dark_one = dark_one
        self._dark_three = dark_three
        self._dark_four = dark_four
        self._bg_one = bg_one
        self._icon_color = icon_color
        self._icon_color_hover = icon_color_hover
        self._icon_color_pressed = icon_color_pressed
        self._icon_color_active = icon_color_active
        self._context_color = context_color
        self._text_foreground = text_foreground
        self._text_active = text_active
        self._duration_time = duration_time
        self._radius = radius
        self._minimum_width = minimum_width
        self._maximum_width = maximum_width
        self._icon_path = Functions.set_svg_icon(icon_path)
        self._icon_path_close = Functions.set_svg_icon(icon_path_close)

        # SET PARENT
        self._parent = parent
        self._app_parent = app_parent

        # SETUP WIDGETS
        self.setup_ui()

        # SET BG COLOR
        self.bg.setStyleSheet(f"background: {dark_one}; border-radius: {radius};")

        # TOGGLE BUTTON AND DIV MENUS
        # ///////////////////////////////////////////////////////////////
        self.toggle_button = PyLeftMenuButton(
            app_parent, 
            text = toggle_text, 
            tooltip_text = toggle_tooltip,
            dark_one = self._dark_one,
            dark_three = self._dark_three,
            dark_four = self._dark_four,
            bg_one = self._bg_one,
            icon_color = self._icon_color,
            icon_color_hover = self._icon_color_active,
            icon_color_pressed = self._icon_color_pressed,
            icon_color_active = self._icon_color_active,
            context_color = self._context_color,
            text_foreground = self._text_foreground,
            text_active = self._text_active,
            icon_path = icon_path
        )
        self.toggle_button.clicked.connect(self.toggle_animation)
        self.div_top = PyDiv(dark_four)

        # ADD TO TOP LAYOUT
        # ///////////////////////////////////////////////////////////////
        self.top_layout.addWidget(self.toggle_button)
        self.top_layout.addWidget(self.div_top)

        # ADD TO BOTTOM LAYOUT
        # ///////////////////////////////////////////////////////////////
        self.div_bottom = PyDiv(dark_four)
        self.div_bottom.hide()
        self.bottom_layout.addWidget(self.div_bottom)

    # ADD BUTTONS TO LEFT MENU
    # Add btns and emit signals
    # ///////////////////////////////////////////////////////////////
    def add_menus(self, parameters):
        if parameters != None:
            for parameter in parameters:
                _btn_icon = parameter['btn_icon']
                _btn_id = parameter['btn_id']
                _btn_text = parameter['btn_text']
                _btn_tooltip = parameter['btn_tooltip']
                _show_top = parameter['show_top']
                _is_active = parameter['is_active']

                self.menu = PyLeftMenuButton(
                    self._app_parent,
                    text = _btn_text,
                    btn_id = _btn_id,
                    tooltip_text = _btn_tooltip,
                    dark_one = self._dark_one,
                    dark_three = self._dark_three,
                    dark_four = self._dark_four,
                    bg_one = self._bg_one,
                    icon_color = self._icon_color,
                    icon_color_hover = self._icon_color_active,
                    icon_color_pressed = self._icon_color_pressed,
                    icon_color_active = self._icon_color_active,
                    context_color = self._context_color,
                    text_foreground = self._text_foreground,
                    text_active = self._text_active,
                    icon_path = _btn_icon,
                    is_active = _is_active
                )
                self.menu.clicked.connect(self.btn_clicked)
                self.menu.released.connect(self.btn_released)

                # ADD TO LAYOUT
                if _show_top:
                    self.top_layout.addWidget(self.menu)
                else:
                    self.div_bottom.show()
                    self.bottom_layout.addWidget(self.menu)

    # LEFT MENU EMIT SIGNALS
    # ///////////////////////////////////////////////////////////////
    def btn_clicked(self):
        self.clicked.emit(self.menu)
    
    def btn_released(self):
        self.released.emit(self.menu)

    # EXPAND / RETRACT LEF MENU
    # ///////////////////////////////////////////////////////////////
    def toggle_animation(self):
        # CREATE ANIMATION
        self.animation = QPropertyAnimation(self._parent, b"minimumWidth")
        self.animation.stop()
        if self.width() == self._minimum_width:
            self.animation.setStartValue(self.width())
            self.animation.setEndValue(self._maximum_width)
            self.toggle_button.set_active_toggle(True)
            self.toggle_button.set_icon(self._icon_path_close)
        else:
            self.animation.setStartValue(self.width())
            self.animation.setEndValue(self._minimum_width)
            self.toggle_button.set_active_toggle(False)
            self.toggle_button.set_icon(self._icon_path)
        self.animation.setEasingCurve(QEasingCurve.InOutCubic)
        self.animation.setDuration(self._duration_time)
        self.animation.start()

    # SELECT ONLY ONE BTN
    # ///////////////////////////////////////////////////////////////
    def select_only_one(self, widget: str):
        for btn in self.findChildren(QPushButton):
            if btn.objectName() == widget:
                btn.set_active(True)
            else:
                btn.set_active(False)

    # SELECT ONLY ONE TAB BTN
    # ///////////////////////////////////////////////////////////////
    def select_only_one_tab(self, widget: str):
        for btn in self.findChildren(QPushButton):
            if btn.objectName() == widget:
                btn.set_active_tab(True)
            else:
                btn.set_active_tab(False)

    # DESELECT ALL BTNs
    # ///////////////////////////////////////////////////////////////
    def deselect_all(self):
        for btn in self.findChildren(QPushButton):
            btn.set_active(False)

    # DESELECT ALL TAB BTNs
    # ///////////////////////////////////////////////////////////////
    def deselect_all_tab(self):
        for btn in self.findChildren(QPushButton):
            btn.set_active_tab(False)

    # SETUP APP
    # ///////////////////////////////////////////////////////////////
    def setup_ui(self):
        # ADD MENU LAYOUT
        self.left_menu_layout = QVBoxLayout(self)
        self.left_menu_layout.setContentsMargins(0,0,0,0)

        # ADD BG
        self.bg = QFrame()

        # TOP FRAME
        self.top_frame = QFrame()

        # BOTTOM FRAME
        self.bottom_frame = QFrame()

        # ADD LAYOUTS
        self._layout = QVBoxLayout(self.bg)
        self._layout.setContentsMargins(0,0,0,0)

        # TOP LAYOUT
        self.top_layout = QVBoxLayout(self.top_frame)
        self.top_layout.setContentsMargins(0,0,0,0)
        self.top_layout.setSpacing(1)

        # BOTTOM LAYOUT
        self.bottom_layout = QVBoxLayout(self.bottom_frame)
        self.bottom_layout.setContentsMargins(0,0,0,8)
        self.bottom_layout.setSpacing(1)

        # ADD TOP AND BOTTOM FRAME
        self._layout.addWidget(self.top_frame, 0, Qt.AlignTop)
        self._layout.addWidget(self.bottom_frame, 0, Qt.AlignBottom)

        # ADD BG TO LAYOUT
        self.left_menu_layout.addWidget(self.bg)

        

        




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'left_column.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_LeftColumn(object):
    def setupUi(self, LeftColumn):
        if not LeftColumn.objectName():
            LeftColumn.setObjectName(u"LeftColumn")
        LeftColumn.resize(240, 600)
        self.main_pages_layout = QVBoxLayout(LeftColumn)
        self.main_pages_layout.setSpacing(0)
        self.main_pages_layout.setObjectName(u"main_pages_layout")
        self.main_pages_layout.setContentsMargins(5, 5, 5, 5)
        self.menus = QStackedWidget(LeftColumn)
        self.menus.setObjectName(u"menus")
        self.menu_1 = QWidget()
        self.menu_1.setObjectName(u"menu_1")
        self.verticalLayout = QVBoxLayout(self.menu_1)
        self.verticalLayout.setSpacing(5)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(5, 5, 5, 5)
        self.btn_1_widget = QWidget(self.menu_1)
        self.btn_1_widget.setObjectName(u"btn_1_widget")
        self.btn_1_widget.setMinimumSize(QSize(0, 40))
        self.btn_1_widget.setMaximumSize(QSize(16777215, 40))
        self.btn_1_layout = QVBoxLayout(self.btn_1_widget)
        self.btn_1_layout.setSpacing(0)
        self.btn_1_layout.setObjectName(u"btn_1_layout")
        self.btn_1_layout.setContentsMargins(0, 0, 0, 0)

        self.verticalLayout.addWidget(self.btn_1_widget)

        self.btn_2_widget = QWidget(self.menu_1)
        self.btn_2_widget.setObjectName(u"btn_2_widget")
        self.btn_2_widget.setMinimumSize(QSize(0, 40))
        self.btn_2_widget.setMaximumSize(QSize(16777215, 40))
        self.btn_2_layout = QVBoxLayout(self.btn_2_widget)
        self.btn_2_layout.setSpacing(0)
        self.btn_2_layout.setObjectName(u"btn_2_layout")
        self.btn_2_layout.setContentsMargins(0, 0, 0, 0)

        self.verticalLayout.addWidget(self.btn_2_widget)

        self.btn_3_widget = QWidget(self.menu_1)
        self.btn_3_widget.setObjectName(u"btn_3_widget")
        self.btn_3_widget.setMinimumSize(QSize(0, 40))
        self.btn_3_widget.setMaximumSize(QSize(16777215, 40))
        self.btn_3_layout = QVBoxLayout(self.btn_3_widget)
        self.btn_3_layout.setSpacing(0)
        self.btn_3_layout.setObjectName(u"btn_3_layout")
        self.btn_3_layout.setContentsMargins(0, 0, 0, 0)

        self.verticalLayout.addWidget(self.btn_3_widget)

        self.label_1 = QLabel(self.menu_1)
        self.label_1.setObjectName(u"label_1")
        font = QFont()
        font.setPointSize(16)
        self.label_1.setFont(font)
        self.label_1.setStyleSheet(u"font-size: 16pt")
        self.label_1.setAlignment(Qt.AlignCenter)

        self.verticalLayout.addWidget(self.label_1)

        self.menus.addWidget(self.menu_1)
        self.menu_2 = QWidget()
        self.menu_2.setObjectName(u"menu_2")
        self.verticalLayout_2 = QVBoxLayout(self.menu_2)
        self.verticalLayout_2.setSpacing(5)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(5, 5, 5, 5)
        self.btn_4_widget = QWidget(self.menu_2)
        self.btn_4_widget.setObjectName(u"btn_4_widget")
        self.btn_4_widget.setMinimumSize(QSize(0, 40))
        self.btn_4_widget.setMaximumSize(QSize(16777215, 40))
        self.btn_4_layout = QVBoxLayout(self.btn_4_widget)
        self.btn_4_layout.setSpacing(0)
        self.btn_4_layout.setObjectName(u"btn_4_layout")
        self.btn_4_layout.setContentsMargins(0, 0, 0, 0)

        self.verticalLayout_2.addWidget(self.btn_4_widget)

        self.label_2 = QLabel(self.menu_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setFont(font)
        self.label_2.setStyleSheet(u"font-size: 16pt")
        self.label_2.setAlignment(Qt.AlignCenter)

        self.verticalLayout_2.addWidget(self.label_2)

        self.label_3 = QLabel(self.menu_2)
        self.label_3.setObjectName(u"label_3")
        font1 = QFont()
        font1.setPointSize(9)
        self.label_3.setFont(font1)
        self.label_3.setStyleSheet(u"font-size: 9pt")
        self.label_3.setAlignment(Qt.AlignCenter)
        self.label_3.setWordWrap(True)

        self.verticalLayout_2.addWidget(self.label_3)

        self.menus.addWidget(self.menu_2)

        self.main_pages_layout.addWidget(self.menus)


        self.retranslateUi(LeftColumn)

        self.menus.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(LeftColumn)
    # setupUi

    def retranslateUi(self, LeftColumn):
        LeftColumn.setWindowTitle(QCoreApplication.translate("LeftColumn", u"Form", None))
        self.label_1.setText(QCoreApplication.translate("LeftColumn", u"Menu 1 - Left Menu", None))
        self.label_2.setText(QCoreApplication.translate("LeftColumn", u"Menu 2 - Left Menu", None))
        self.label_3.setText(QCoreApplication.translate("LeftColumn", u"This is just an example menu.\n"
"Add Qt Widgets or your custom widgets here.", None))
    # retranslateUi




# STYLE
# ///////////////////////////////////////////////////////////////
style = '''
/* /////////////////////////////////////////////////////////////////////////////////////////////////
QTableWidget */

QTableWidget {{	
	background-color: {_bg_color};
	padding: 5px;
	border-radius: {_radius}px;
	gridline-color: {_grid_line_color};
    color: {_color};
}}
QTableWidget::item{{
	border-color: none;
	padding-left: 5px;
	padding-right: 5px;
	gridline-color: rgb(44, 49, 60);
    border-bottom: 1px solid {_bottom_line_color};
}}
QTableWidget::item:selected{{
	background-color: {_selection_color};
}}
QHeaderView::section{{
	background-color: rgb(33, 37, 43);
	max-width: 30px;
	border: 1px solid rgb(44, 49, 58);
	border-style: none;
    border-bottom: 1px solid rgb(44, 49, 60);
    border-right: 1px solid rgb(44, 49, 60);
}}
QTableWidget::horizontalHeader {{	
	background-color: rgb(33, 37, 43);
}}
QTableWidget QTableCornerButton::section {{
    border: none;
	background-color: {_header_horizontal_color};
	padding: 3px;
    border-top-left-radius: {_radius}px;
}}
QHeaderView::section:horizontal
{{
    border: none;
	background-color: {_header_horizontal_color};
	padding: 3px;
}}
QHeaderView::section:vertical
{{
    border: none;
	background-color: {_header_vertical_color};
	padding-left: 5px;
    padding-right: 5px;
    border-bottom: 1px solid {_bottom_line_color};
    margin-bottom: 1px;
}}


/* /////////////////////////////////////////////////////////////////////////////////////////////////
ScrollBars */
QScrollBar:horizontal {{
    border: none;
    background: {_scroll_bar_bg_color};
    height: 8px;
    margin: 0px 21px 0 21px;
	border-radius: 0px;
}}
QScrollBar::handle:horizontal {{
    background: {_context_color};
    min-width: 25px;
	border-radius: 4px
}}
QScrollBar::add-line:horizontal {{
    border: none;
    background: {_scroll_bar_btn_color};
    width: 20px;
	border-top-right-radius: 4px;
    border-bottom-right-radius: 4px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}}
QScrollBar::sub-line:horizontal {{
    border: none;
    background: {_scroll_bar_btn_color};
    width: 20px;
	border-top-left-radius: 4px;
    border-bottom-left-radius: 4px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}}
QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal
{{
     background: none;
}}
QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal
{{
     background: none;
}}
QScrollBar:vertical {{
	border: none;
    background: {_scroll_bar_bg_color};
    width: 8px;
    margin: 21px 0 21px 0;
	border-radius: 0px;
}}
QScrollBar::handle:vertical {{	
	background: {_context_color};
    min-height: 25px;
	border-radius: 4px
}}
QScrollBar::add-line:vertical {{
     border: none;
    background: {_scroll_bar_btn_color};
     height: 20px;
	border-bottom-left-radius: 4px;
    border-bottom-right-radius: 4px;
     subcontrol-position: bottom;
     subcontrol-origin: margin;
}}
QScrollBar::sub-line:vertical {{
	border: none;
    background: {_scroll_bar_btn_color};
     height: 20px;
	border-top-left-radius: 4px;
    border-top-right-radius: 4px;
     subcontrol-position: top;
     subcontrol-origin: margin;
}}
QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {{
     background: none;
}}

QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {{
     background: none;
}}
'''


# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# STYLE
# ///////////////////////////////////////////////////////////////
style = '''
QLineEdit {{
	background-color: {_bg_color};
	border-radius: {_radius}px;
	border: {_border_size}px solid transparent;
	padding-left: 10px;
    padding-right: 10px;
	selection-color: {_selection_color};
	selection-background-color: {_context_color};
    color: {_color};
}}
QLineEdit:focus {{
	border: {_border_size}px solid {_context_color};
    background-color: {_bg_color_active};
}}
'''

# PY PUSH BUTTON
# ///////////////////////////////////////////////////////////////
class PyLineEdit(QLineEdit):
    def __init__(
        self, 
        text = "",
        place_holder_text = "",
        radius = 8,
        border_size = 2,
        color = "#FFF",
        selection_color = "#FFF",
        bg_color = "#333",
        bg_color_active = "#222",
        context_color = "#00ABE8"
    ):
        super().__init__()

        # PARAMETERS
        if text:
            self.setText(text)
        if place_holder_text:
            self.setPlaceholderText(place_holder_text)

        # SET STYLESHEET
        self.set_stylesheet(
            radius,
            border_size,
            color,
            selection_color,
            bg_color,
            bg_color_active,
            context_color
        )

    # SET STYLESHEET
    def set_stylesheet(
        self,
        radius,
        border_size,
        color,
        selection_color,
        bg_color,
        bg_color_active,
        context_color
    ):
        # APPLY STYLESHEET
        style_format = style.format(
            _radius = radius,
            _border_size = border_size,           
            _color = color,
            _selection_color = selection_color,
            _bg_color = bg_color,
            _bg_color_active = bg_color_active,
            _context_color = context_color
        )
        self.setStyleSheet(style_format)


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'right_column.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


from consolewidget import ConsoleWidget


class Ui_RightColumn(object):
    def setupUi(self, RightColumn):
        if not RightColumn.objectName():
            RightColumn.setObjectName(u"RightColumn")
        RightColumn.resize(240, 600)
        self.main_pages_layout = QVBoxLayout(RightColumn)
        self.main_pages_layout.setSpacing(0)
        self.main_pages_layout.setObjectName(u"main_pages_layout")
        self.main_pages_layout.setContentsMargins(5, 5, 5, 5)
        self.menus = QStackedWidget(RightColumn)
        self.menus.setObjectName(u"menus")
        self.menu_1 = QWidget()
        self.menu_1.setObjectName(u"menu_1")
        self.verticalLayout = QVBoxLayout(self.menu_1)
        self.verticalLayout.setSpacing(5)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(5, 5, 5, 5)

        self.btn_1_widget = QWidget(self.menu_1)
        self.btn_1_widget.setObjectName(u"btn_1_widget")
        self.btn_1_widget.setMinimumSize(QSize(0, 40))
        self.btn_1_widget.setMaximumSize(QSize(16777215, 40))
        self.btn_1_layout = QVBoxLayout(self.btn_1_widget)
        self.btn_1_layout.setSpacing(0)
        self.btn_1_layout.setObjectName(u"btn_1_layout")
        self.btn_1_layout.setContentsMargins(0, 0, 0, 0)

        self.verticalLayout.addWidget(self.btn_1_widget)

        ################ Add console 1 by code
        self.ConsoleWidget_1 = ConsoleWidget(self.menu_1)
        self.ConsoleWidget_1.setObjectName(u"ConsoleWidget_1")
        self.ConsoleWidget_1.setEnabled(True)
        self.ConsoleWidget_1.setMinimumSize(QSize(500, 300))
        self.ConsoleWidget_1.setStyleSheet(u"background-color: rgb(85, 0, 127);")
        self.verticalLayout.addWidget(self.ConsoleWidget_1)
        ##############

        self.label_1 = QLabel(self.menu_1)
        self.label_1.setObjectName(u"label_1")
        font = QFont()
        font.setPointSize(16)
        self.label_1.setFont(font)
        self.label_1.setStyleSheet(u"font-size: 16pt")
        self.label_1.setAlignment(Qt.AlignCenter)

        self.verticalLayout.addWidget(self.label_1)

        self.menus.addWidget(self.menu_1)
        self.menu_2 = QWidget()
        self.menu_2.setObjectName(u"menu_2")
        self.verticalLayout_2 = QVBoxLayout(self.menu_2)
        self.verticalLayout_2.setSpacing(5)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(5, 5, 5, 5)

        self.btn_2_widget = QWidget(self.menu_2)
        self.btn_2_widget.setObjectName(u"btn_2_widget")
        self.btn_2_widget.setMinimumSize(QSize(0, 40))
        self.btn_2_widget.setMaximumSize(QSize(16777215, 40))
        self.btn_2_layout = QVBoxLayout(self.btn_2_widget)
        self.btn_2_layout.setSpacing(0)
        self.btn_2_layout.setObjectName(u"btn_2_layout")
        self.btn_2_layout.setContentsMargins(0, 0, 0, 0)

        self.verticalLayout_2.addWidget(self.btn_2_widget)


        self.label_2 = QLabel(self.menu_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setFont(font)
        self.label_2.setStyleSheet(u"font-size: 16pt")
        self.label_2.setAlignment(Qt.AlignCenter)

        self.verticalLayout_2.addWidget(self.label_2)

        # add menu 2
        self.menus.addWidget(self.menu_2)

        ################ Add console 2 by code
        # self.ConsoleWidget_2 = ConsoleWidget(self.menu_2)
        # self.ConsoleWidget_2.setObjectName(u"ConsoleWidget_2")
        # self.ConsoleWidget_2.setEnabled(True)
        # self.ConsoleWidget_2.setMinimumSize(QSize(500, 300))
        # self.ConsoleWidget_2.setStyleSheet(u"background-color: rgb(85, 0, 127);")
        # self.verticalLayout_2.addWidget(self.ConsoleWidget_2)
        ##############


        self.main_pages_layout.addWidget(self.menus)


        self.retranslateUi(RightColumn)

        self.menus.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(RightColumn)
    # setupUi

    def retranslateUi(self, RightColumn):
        RightColumn.setWindowTitle(QCoreApplication.translate("RightColumn", u"Form", None))
        self.label_1.setText(QCoreApplication.translate("RightColumn", u"Menu 1 - Right Menu", None))
        self.label_2.setText(QCoreApplication.translate("RightColumn", u"Menu 2 - Right Menu", None))
    # retranslateUi




# %%
from pylab import np, plt, mpl
plt.style.use('ggplot')

mpl.rcParams['legend.fontsize'] = 10
mpl.rcParams['font.family'] = ['serif'] # default is sans-serif
mpl.rcParams['font.serif'] = ['Times New Roman']
font = {'family':'Times New Roman', 'weight':'normal', 'size':10}

# this changes font for ticks for my TEC-ISMB paper figures but has no effect here, why?
mpl.rcParams['mathtext.fontset'] = 'stix'
mpl.rcParams['font.family'] = ['STIXGeneral', 'Times New Roman']
mpl.rcParams['font.size'] = 14.0
mpl.rcParams['legend.fontsize'] = 12.5

plt.style.use('ggplot') 



''' Current Function
'''
I_AMPL = 5.0
I_AMPL = 0.2
# I_AMPL = 1.2

PERIOD = 0.1 # s
CL_TS = 1e-4
time = np.arange(0, PERIOD, CL_TS)
phase_angle = np.arange(-np.pi, np.pi, 2*np.pi / len(time))
phase_current = np.sin(phase_angle) * I_AMPL

""" Compensation Voltage Function By Current Value """
def sigmoid(x, a1=0, a2=2, a3=10):
    return a1 * x + a2 / (1.0 + np.exp(-a3 * x)) - a2*0.5

def basic_drawings(cv_function):
    # u-i curve quadratic
    ui_curve = [cv_function(current_value) for current_value in phase_current]

    print('Figure: UI-Curve')
    fig_UIcurve = plt.figure(figsize=(20,6), facecolor='w')
    fig_UIcurve.gca().plot(phase_current, ui_curve)

    print('Figure: Time Domain')
    fig_timeDomain = plt.figure(figsize=(20,6), facecolor='w')
    fig_timeDomain.gca().plot(time, ui_curve)
    fig_timeDomain.gca().plot(time, phase_current)

    return ui_curve

ui_curve = basic_drawings(cv_function=sigmoid)


# %%
import sys
sys.path.insert(1, r'D:\DrH\Codes\visualize')
from DFT_CJH import myDFT_single_channel, DFT_CJH

REPEAT = 10

def basic_DFT_analysis(raw_data):

    time = np.arange(0, REPEAT*PERIOD, CL_TS)

    signal = np.array(REPEAT*list(raw_data))
    print('Figure: DFT')
    fig, ax_matrix = plt.subplots(ncols=1, nrows=2, sharex=True, figsize=(20,6))
    ax1 = ax_matrix[0]
    ax2 = ax_matrix[1]
    ax1.set_xlabel('Frequency [Hz]')
    ax1.set_ylabel('Amplitude [dB]')
    ax2.set_xlabel('Frequency [Hz]')
    ax2.set_ylabel('Phase [deg]')

    DFT_CJH(ax1, time, signal, bool_use_dB=False, ax2=ax2)

    # ax1.set_xlim([0,2000])
    ax1.set_xlim([0,200])
    # ax.set_ylim([-50,5])

    print(len(time), len(signal))

basic_DFT_analysis(ui_curve)

# %%
REPEAT = 1
time = np.arange(0, REPEAT*PERIOD, CL_TS)

# Free parameters
OPTION = 1
if OPTION == 1: # I_plateau is dependent parameter
    LIST_PARAM = np.arange(0.5, 10, 0.5)

def parametric_analysis_of_harmonic_content(ui_curve):

    fig_UIcurve.gca().plot(phase_current, ui_curve)
    fig_timeDomain.gca().plot(time, ui_curve)

    signal = np.array(REPEAT*list(ui_curve))
    dft_dict = myDFT_single_channel(time, signal)

    # print([el for el in dft_dict['ampl'][0:15]])
    # print([el for el in dft_dict['ampl'][0:15] if el>1e-5])

    if REPEAT == 1: # 如果REPEAT变了分辨率就变了，间隔也就变了哦。
        harmonics[ 1].append(dft_dict['ampl'][ 1]); #print(' 10 Hz', dft_dict['ampl'][ 1])
        harmonics[ 5].append(dft_dict['ampl'][ 5]); #print(' 50 Hz', dft_dict['ampl'][ 5])
        harmonics[ 7].append(dft_dict['ampl'][ 7]); #print(' 70 Hz', dft_dict['ampl'][ 7])
        harmonics[11].append(dft_dict['ampl'][11]); #print('110 Hz', dft_dict['ampl'][11])
        harmonics[13].append(dft_dict['ampl'][13]); #print('130 Hz', dft_dict['ampl'][13])

harmonics = dict()
harmonics[1] = []
harmonics[5] = []
harmonics[7] = []
harmonics[11] = []
harmonics[13] = []
fig_UIcurve = plt.figure(figsize=(20,6), facecolor='w')
fig_timeDomain = plt.figure(figsize=(20,6), facecolor='w')
for a3 in LIST_PARAM:

    # Dependent parameters
    if OPTION == 1:
        pass

    ui_curve = [sigmoid(current_value, a3=a3) for current_value in phase_current]
    parametric_analysis_of_harmonic_content(ui_curve)

plt.figure(figsize=(20,6), facecolor='w')
plt.plot(harmonics[ 1], '.-')
plt.figure(figsize=(20,6), facecolor='w')
plt.plot(harmonics[ 5], '.-')
plt.plot(harmonics[ 7], '.-')
plt.plot(harmonics[11], '.-')
plt.plot(harmonics[13], '.-')
plt.show()

# %%

REPEAT = 10

""" a3 错 """
ui_curve_1 = [sigmoid(current_value, a2=2, a3=10) for current_value in phase_current]
ui_curve_2 = [sigmoid(current_value, a2=2, a3=20) for current_value in phase_current]
basic_DFT_analysis(np.array(ui_curve_1) - np.array(ui_curve_2))

ui_curve_1 = [sigmoid(current_value, a2=2, a3=10) for current_value in phase_current]
ui_curve_2 = [sigmoid(current_value, a2=2, a3=5) for current_value in phase_current]
basic_DFT_analysis(np.array(ui_curve_1) - np.array(ui_curve_2))

'''还要注意到，a3变化所产生的效果是非线性的，当a3小到一定程度时，对谐波含量的影响会变大！'''

""" a2 错 """
ui_curve_1 = [sigmoid(current_value, a2=2, a3=10) for current_value in phase_current]
ui_curve_2 = [sigmoid(current_value, a2=2.5, a3=10) for current_value in phase_current]
basic_DFT_analysis(np.array(ui_curve_1) - np.array(ui_curve_2))

ui_curve_1 = [sigmoid(current_value, a2=2, a3=10) for current_value in phase_current]
ui_curve_2 = [sigmoid(current_value, a2=1.5, a3=10) for current_value in phase_current]
basic_DFT_analysis(np.array(ui_curve_1) - np.array(ui_curve_2))


''' 
初步结论，如果想辨识a3，最好用高次谐波电流去做检测，
因为a2误差导致的误差中除了基波含量以外最大的就是六次。
如果你用六次去校正a3，那a2的误差也会被你吃进去。
'''


# %%
ui_curve_1 = [sigmoid(current_value, a2=2, a3=10) for current_value in phase_current]
ui_curve_2 = [sigmoid(current_value, a2=1.8, a3=5) for current_value in phase_current]
raw_data = np.array(ui_curve_1) - np.array(ui_curve_2)
signal = np.array(REPEAT*list(raw_data))
time = np.arange(0, REPEAT*PERIOD, CL_TS)
dft_dict = myDFT_single_channel(
    time=time,
    signal=signal,
    bool_use_NFFT=False
)

# %%



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT PACKAGES AND MODULES
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# IMPORT SETTINGS
# ///////////////////////////////////////////////////////////////
from gui.core.json_settings import Settings

# IMPORT STYLES
# ///////////////////////////////////////////////////////////////
from . styles import Styles

# PY WINDOW
# ///////////////////////////////////////////////////////////////
class PyWindow(QFrame):
    def __init__(
        self,
        parent,
        layout = Qt.Vertical,
        margin = 0,
        spacing = 2,
        bg_color = "#2c313c",
        text_color = "#fff",
        text_font = "9pt 'Segoe UI'",
        border_radius = 10,
        border_size = 2,
        border_color = "#343b48",
        enable_shadow = True
    ):
        super().__init__()

        # LOAD SETTINGS
        # ///////////////////////////////////////////////////////////////
        settings = Settings()
        self.settings = settings.items

        # PROPERTIES
        # ///////////////////////////////////////////////////////////////
        self.parent = parent
        self.layout = layout
        self.margin = margin
        self.bg_color = bg_color
        self.text_color = text_color
        self.text_font = text_font
        self.border_radius = border_radius
        self.border_size = border_size
        self.border_color = border_color
        self.enable_shadow = enable_shadow

        # OBJECT NAME
        # ///////////////////////////////////////////////////////////////   
        self.setObjectName("pod_bg_app")

        # APPLY STYLESHEET
        # /////////////////////////////////////////////////////////////// 
        self.set_stylesheet()

        # ADD LAYOUT
        # ///////////////////////////////////////////////////////////////
        if layout == Qt.Vertical:
            # VERTICAL LAYOUT
            self.layout = QHBoxLayout(self)
        else:
            # HORIZONTAL LAYOUT
            self.layout = QHBoxLayout(self)
        self.layout.setContentsMargins(margin, margin, margin, margin)
        self.layout.setSpacing(spacing)

        # ADD DROP SHADOW
        # ///////////////////////////////////////////////////////////////
        if self.settings["custom_title_bar"]:
            if enable_shadow:
                self.shadow = QGraphicsDropShadowEffect()
                self.shadow.setBlurRadius(20)
                self.shadow.setXOffset(0)
                self.shadow.setYOffset(0)
                self.shadow.setColor(QColor(0, 0, 0, 160))
                self.setGraphicsEffect(self.shadow)

    # APPLY AND UPDATE STYLESHEET
    # ///////////////////////////////////////////////////////////////
    def set_stylesheet(
        self,
        bg_color = None,
        border_radius = None,
        border_size = None,
        border_color = None,
        text_color = None,
        text_font = None
    ):
        # CHECK BG COLOR
        if bg_color != None: internal_bg_color = bg_color
        else: internal_bg_color = self.bg_color

        # CHECK BORDER RADIUS
        if border_radius != None: internal_border_radius = border_radius
        else: internal_border_radius = self.border_radius

        # CHECK BORDER SIZE
        if border_size != None: internal_border_size = border_size
        else: internal_border_size = self.border_size

        # CHECK BORDER COLOR
        if text_color != None: internal_text_color = text_color
        else: internal_text_color = self.text_color

        # CHECK TEXT COLOR
        if border_color != None: internal_border_color = border_color
        else: internal_border_color = self.border_color

        # CHECK TEXT COLOR
        if text_font != None: internal_text_font = text_font
        else: internal_text_font = self.text_font

        self.setStyleSheet(Styles.bg_style.format(
            _bg_color = internal_bg_color,
            _border_radius = internal_border_radius,
            _border_size = internal_border_size,
            _border_color = internal_border_color,
            _text_color = internal_text_color,
            _text_font = internal_text_font
        ))
        


# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT PACKAGES AND MODULES
# ///////////////////////////////////////////////////////////////
import sys

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# LOAD UI MAIN
# ///////////////////////////////////////////////////////////////
from . ui_main import *

# FUNCTIONS
class MainFunctions():
    def __init__(self):
        super().__init__()
        # SETUP MAIN WINDOw
        # Load widgets from "gui\uis\main_window\ui_main.py"
        # ///////////////////////////////////////////////////////////////
        self.ui = UI_MainWindow()
        self.ui.setup_ui(self)

    # SET MAIN WINDOW PAGES
    # ///////////////////////////////////////////////////////////////
    def set_page(self, page):
        self.ui.load_pages.pages.setCurrentWidget(page)

    # SET LEFT COLUMN PAGES
    # ///////////////////////////////////////////////////////////////
    def set_left_column_menu(
        self,
        menu,
        title,
        icon_path
    ):
        self.ui.left_column.menus.menus.setCurrentWidget(menu)
        self.ui.left_column.title_label.setText(title)
        self.ui.left_column.icon.set_icon(icon_path)

    # RETURN IF LEFT COLUMN IS VISIBLE
    # ///////////////////////////////////////////////////////////////
    def left_column_is_visible(self):
        width = self.ui.left_column_frame.width()
        if width == 0:
            return False
        else:
            return True

    # RETURN IF RIGHT COLUMN IS VISIBLE
    # ///////////////////////////////////////////////////////////////
    def right_column_is_visible(self):
        width = self.ui.right_column_frame.width()
        if width == 0:
            return False
        else:
            return True

    # SET RIGHT COLUMN PAGES
    # ///////////////////////////////////////////////////////////////
    def set_right_column_menu(self, menu):
        self.ui.right_column.menus.setCurrentWidget(menu)

    # GET TITLE BUTTON BY OBJECT NAME
    # ///////////////////////////////////////////////////////////////
    def get_title_bar_btn(self, object_name):
        return self.ui.title_bar_frame.findChild(QPushButton, object_name)

    # GET TITLE BUTTON BY OBJECT NAME
    # ///////////////////////////////////////////////////////////////
    def get_left_menu_btn(self, object_name):
        return self.ui.left_menu.findChild(QPushButton, object_name)
    
    # LEDT AND RIGHT COLUMNS / SHOW / HIDE
    # ///////////////////////////////////////////////////////////////
    def toggle_left_column(self):
        # GET ACTUAL CLUMNS SIZE
        width = self.ui.left_column_frame.width()
        right_column_width = self.ui.right_column_frame.width()

        MainFunctions.start_box_animation(self, width, right_column_width, "left")

    def toggle_right_column(self):
        # GET ACTUAL CLUMNS SIZE
        left_column_width = self.ui.left_column_frame.width()
        width = self.ui.right_column_frame.width()

        MainFunctions.start_box_animation(self, left_column_width, width, "right")

    def start_box_animation(self, left_box_width, right_box_width, direction):
        right_width = 0
        left_width = 0
        time_animation = self.ui.settings["time_animation"]
        minimum_left = self.ui.settings["left_column_size"]["minimum"]
        maximum_left = self.ui.settings["left_column_size"]["maximum"]
        minimum_right = self.ui.settings["right_column_size"]["minimum"]
        maximum_right = self.ui.settings["right_column_size"]["maximum"]

        # Check Left Values        
        if left_box_width == minimum_left and direction == "left":
            left_width = maximum_left
        else:
            left_width = minimum_left

        # Check Right values        
        if right_box_width == minimum_right and direction == "right":
            right_width = maximum_right
        else:
            right_width = minimum_right       

        # ANIMATION LEFT BOX        
        self.left_box = QPropertyAnimation(self.ui.left_column_frame, b"minimumWidth")
        self.left_box.setDuration(time_animation)
        self.left_box.setStartValue(left_box_width)
        self.left_box.setEndValue(left_width)
        self.left_box.setEasingCurve(QEasingCurve.InOutQuart)

        # ANIMATION RIGHT BOX        
        self.right_box = QPropertyAnimation(self.ui.right_column_frame, b"minimumWidth")
        self.right_box.setDuration(time_animation)
        self.right_box.setStartValue(right_box_width)
        self.right_box.setEndValue(right_width)
        self.right_box.setEasingCurve(QEasingCurve.InOutQuart)

        # GROUP ANIMATION
        self.group = QParallelAnimationGroup()
        self.group.stop()
        self.group.addAnimation(self.left_box)
        self.group.addAnimation(self.right_box)
        self.group.start()




# from app_modules import *
# from PySide6 import QtCore, QtGui, QtWidgets
# from PySide6.QtCore import (QCoreApplication, QPropertyAnimation, QDate, QDateTime, QMetaObject, QObject, QPoint, QRect, QSize, QTime, QUrl, Qt, QEvent, Signal, QEasingCurve, QTimer)
# from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont, QFontDatabase, QIcon, QKeySequence, QLinearGradient, QPalette, QPainter, QPixmap, QRadialGradient, QPen, QPainterPath, QImage)
from PySide2.QtWidgets import QApplication, QMainWindow, QPushButton, QSizePolicy, QWidget, QLineEdit
from PySide2.QtWidgets import QStyleOption, QStyle
from PySide2 import QtGui, QtCore

# from main import *
# if bool_use_PyQt5:
#     from PyQt5 import QtGui, QtCore
#     from PyQt5.QtWidgets import QApplication, QWidget, QStyleOption, QStyle
# else:
#     from PySide2 import QtGui, QtCore
#     from PySide2.QtWidgets import QApplication, QWidget, QStyleOption, QStyle


# https://stackoverflow.com/questions/11513132/embedding-ipython-qt-console-in-a-pyqt-application
# from qtconsole.qt import QtGui # This works with qtconsole-4.6.0
from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager

# import logging

class ConsoleWidget(RichJupyterWidget):

    # def __init__(self, customBanner=None, *args, **kwargs):
    def __init__(self, parent=None, *args, **kwargs):

        # print('DeBUG consolewidget:', parent)

        # QWidget.__init__(self, parent)
        r''' I have to comment this out because of following error:
            D:\DrH\Codes\emachineryTestPYPI\emachinery\guiv2>python main.py
            Traceback (most recent call last):
              File "main.py", line 289, in <module>
                window = MainWindow() # ui=QUiLoader().load("GUI_BASE.ui", None)
              File "main.py", line 25, in __init__
                self.ui.setupUi(self)
              File "D:\DrH\Codes\emachineryTestPYPI\emachinery\guiv2\ui_main.py", line 1294, in setupUi
                self.ConsoleWidget = ConsoleWidget(self.page_namePlateData)
              File "D:\DrH\Codes\emachineryTestPYPI\emachinery\guiv2\consolewidget.py", line 16, in __init__
                QWidget.__init__(self, parent)
            RuntimeError: You can't initialize an object twice!
            QBasicTimer::start: QBasicTimer can only be used with threads started with QThread
        '''

        super(ConsoleWidget, self).__init__(*args, **kwargs)

        # useless for guiv2?
        # self.setStyleSheet("background: rgb(27, 29, 35);")

            #         css = '''
            # color: blue;
            # background-color: yellow;
            # selection-color: yellow;
            # selection-background-color: blue;
            # '''
            #         self.setStyleSheet(css)

        # if customBanner is not None:
        #     self.banner = customBanner
        self.banner = "This is an embedded qtconsole.\nSuggested commands:\n\tvars(emy)\n\n" \
                    + '''You will have access to CONSOLE, ACM, CTRL, HM after you start python+numba based simulation.
\nSome ideas: 
\tHM.CMD_SPEED_SINE_HZ = 20
\treg_speed.OutLimit*=2
\t_dir(CONSOLE)
\t    Out[3]: 
\t    ['CL_TS',
\t    'NUMBER_OF_SAMPLE_TO_SHOW',
\t    'SAMPLING_RATE',
\t    'TIME_SLICE',
\t    'anim',
\t    'offset_anime_ii',
\t    'reset']
\tCONSOLE.NUMBER_OF_SAMPLE_TO_SHOW=6000
\tACM.TLoad=2
\nFinally, you can click on the canvas to stop/start the animation.
\n
'''
                    # [attr for attr in dir(emy) if not callable(getattr(emy, attr)) and not attr.startswith('__')]
                    # https://stackoverflow.com/questions/1398022/looping-over-all-member-variables-of-a-class-in-python

        # 无法修改字体大小
        self.font_size = 6+3
        # self.font_sizeInt = 9+3
        # print(self.font)
        self.font.setPointSize(24) # https://doc.qt.io/qt-5/qfont.html#setPointSize

        self.kernel_manager = kernel_manager = QtInProcessKernelManager()
        kernel_manager.start_kernel(show_banner=False)
        # kernel_manager.kernel.log.setLevel(logging.CRITICAL) #
        kernel_manager.kernel.gui = 'qt'
        self.kernel_client = kernel_client = self._kernel_manager.client()
        kernel_client.start_channels()

        def stop():
            kernel_client.stop_channels()
            kernel_manager.shutdown_kernel()
            guisupport.get_app_qt().exit()
        self.exit_requested.connect(stop)

        # # useless to set background color for promoted widget
        # self.setAttribute(QtCore.Qt.WA_StyledBackground, True) # https://stackoverflow.com/questions/54965088/pyqt-promoted-widget-background-issues?noredirect=1&lq=1

        self.execute_command('''_dir = lambda obj: [method for method in dir(obj) if not method.startswith('__')]''')

    """ Do we need to comment out this function? """
    def paintEvent(self, evt):
        # no effect???
        # Stefan Reinhardt from https://stackoverflow.com/questions/7276330/qt-stylesheet-for-custom-widget
        super(ConsoleWidget, self).paintEvent(evt)
        opt = QStyleOption()
        opt.initFrom(self)
        opt.init(self)
        p = QtGui.QPainter(self)
        s = self.style()
        s.drawPrimitive(QStyle.PE_Widget, opt, p, self) 

    def push_vars(self, variableDict):
        """
        Given a dictionary containing name / value pairs, push those variables
        to the Jupyter console widget
        """
        self.kernel_manager.kernel.shell.push(variableDict)

    def clear(self):
        """
        Clears the terminal
        """
        self._control.clear()

        # self.kernel_manager

    def print_text(self, text):
        """
        Prints some plain text to the console
        """
        self._append_plain_text(text)

    def execute_command(self, command):
        """
        Execute a command in the frame of the console widget
        """
        self._execute(command, False)


if __name__ == '__main__':

    app = QApplication([])
    widget = ConsoleWidget()
    # widget.setStyleSheet("background: rgb(27, 29, 35);")
    widget.show()
    if bool_use_PyQt5:
        app.exec_()
    else:
        sys.exit(app.exec_())



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# PY CREDITS BAR AND VERSION
# ///////////////////////////////////////////////////////////////
class PyCredits(QWidget):
    def __init__(
        self,
        copyright,
        version,
        bg_two,
        font_family,
        text_size,
        text_description_color,
        radius = 8,
        padding = 10
    ):
        super().__init__()

        # PROPERTIES
        self._copyright = copyright
        self._version = version
        self._bg_two = bg_two
        self._font_family = font_family
        self._text_size = text_size
        self._text_description_color = text_description_color
        self._radius = radius
        self._padding = padding

        # SETUP UI
        self.setup_ui()

    def setup_ui(self):
        # ADD LAYOUT
        self.widget_layout = QHBoxLayout(self)
        self.widget_layout.setContentsMargins(0,0,0,0)

        # BG STYLE
        style = f"""
        #bg_frame {{
            border-radius: {self._radius}px;
            background-color: {self._bg_two};
        }}
        .QLabel {{
            font: {self._text_size}pt "{self._font_family}";
            color: {self._text_description_color};
            padding-left: {self._padding}px;
            padding-right: {self._padding}px;
        }}
        """

        # BG FRAME
        self.bg_frame = QFrame()
        self.bg_frame.setObjectName("bg_frame")
        self.bg_frame.setStyleSheet(style)

        # ADD TO LAYOUT
        self.widget_layout.addWidget(self.bg_frame)

        # ADD BG LAYOUT
        self.bg_layout = QHBoxLayout(self.bg_frame)
        self.bg_layout.setContentsMargins(0,0,0,0)

        # ADD COPYRIGHT TEXT
        self.copyright_label = QLabel(self._copyright)
        self.copyright_label.setAlignment(Qt.AlignVCenter)

        # ADD VERSION TEXT
        self.version_label = QLabel(self._version)
        self.version_label.setAlignment(Qt.AlignVCenter)

        # SEPARATOR
        self.separator = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        # ADD TO LAYOUT
        self.bg_layout.addWidget(self.copyright_label)
        self.bg_layout.addSpacerItem(self.separator)
        self.bg_layout.addWidget(self.version_label)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# IMPORT CLOSE BUTTON
# ///////////////////////////////////////////////////////////////
from . py_left_button import *

# IMPORT ICON
# ///////////////////////////////////////////////////////////////
from . py_icon import *

# IMPORT LEFT COLUMN
# ///////////////////////////////////////////////////////////////
from gui.uis.columns.ui_left_column import Ui_LeftColumn

class PyLeftColumn(QWidget):
    # SIGNALS
    clicked = Signal(object)
    released = Signal(object)

    def __init__(
        self,
        parent,
        app_parent,
        text_title,
        text_title_size,
        text_title_color,
        dark_one,
        bg_color,
        btn_color,
        btn_color_hover,
        btn_color_pressed,
        icon_path,
        icon_color,
        icon_color_hover,
        icon_color_pressed,
        context_color,
        icon_close_path,
        radius = 8
    ):
        super().__init__()

        # PARAMETERS
        self._parent = parent
        self._app_parent = app_parent
        self._text_title = text_title
        self._text_title_size = text_title_size
        self._text_title_color = text_title_color
        self._icon_path = icon_path
        self._dark_one = dark_one
        self._bg_color = bg_color
        self._btn_color = btn_color
        self._btn_color_hover = btn_color_hover
        self._btn_color_pressed = btn_color_pressed
        self._icon_color = icon_color
        self._icon_color_hover = icon_color_hover
        self._icon_color_pressed = icon_color_pressed
        self._context_color = context_color
        self._icon_close_path = icon_close_path
        self._radius = radius

        # SETUP UI
        self.setup_ui()

        # ADD LEFT COLUMN TO BG FRAME
        self.menus = Ui_LeftColumn()
        self.menus.setupUi(self.content_frame)

        # CONNECT SIGNALS
        self.btn_close.clicked.connect(self.btn_clicked)
        self.btn_close.released.connect(self.btn_released)

    # TITLE LEFT COLUMN EMIT SIGNALS
    # ///////////////////////////////////////////////////////////////
    def btn_clicked(self):
        self.clicked.emit(self.btn_close)
    
    def btn_released(self):
        self.released.emit(self.btn_close)

    # WIDGETS
    # ///////////////////////////////////////////////////////////////
    def setup_ui(self):
        # BASE LAYOUT
        self.base_layout = QVBoxLayout(self)
        self.base_layout.setContentsMargins(0,0,0,0)
        self.base_layout.setSpacing(0)

        # TITLE FRAME
        # ///////////////////////////////////////////////////////////////
        self.title_frame = QFrame()
        self.title_frame.setMaximumHeight(47)
        self.title_frame.setMinimumHeight(47)

        # TITLE BASE LAYOUT 
        self.title_base_layout = QVBoxLayout(self.title_frame)
        self.title_base_layout.setContentsMargins(5,3,5,3)

        # TITLE BG
        self.title_bg_frame = QFrame()
        self.title_bg_frame.setObjectName("title_bg_frame")
        self.title_bg_frame.setStyleSheet(f'''
        #title_bg_frame {{
            background-color: {self._bg_color};
            border-radius: {self._radius}px;
        }}
        ''')

        # LAYOUT TITLE BG
        self.title_bg_layout = QHBoxLayout(self.title_bg_frame)
        self.title_bg_layout.setContentsMargins(5,5,5,5)
        self.title_bg_layout.setSpacing(3)

        # ICON
        self.icon_frame = QFrame()
        self.icon_frame.setFixedSize(30,30)
        self.icon_frame.setStyleSheet("background: none;")
        self.icon_layout = QVBoxLayout(self.icon_frame)
        self.icon_layout.setContentsMargins(0,0,0,0)
        self.icon_layout.setSpacing(5)
        self.icon = PyIcon(self._icon_path, self._icon_color)
        self.icon_layout.addWidget(self.icon, Qt.AlignCenter, Qt.AlignCenter)

        # LABEL
        self.title_label = QLabel(self._text_title)
        self.title_label.setObjectName("title_label")
        self.title_label.setStyleSheet(f'''
        #title_label {{
            font-size: {self._text_title_size}pt;
            color: {self._text_title_color};
            padding-bottom: 2px;
            background: none;
        }}
        ''')

        # BTN FRAME
        self.btn_frame = QFrame()
        self.btn_frame.setFixedSize(30,30)
        self.btn_frame.setStyleSheet("background: none;")
        # CLOSE BUTTON
        self.btn_close = PyLeftButton(
            self._parent,
            self._app_parent,
            tooltip_text = "Hide",
            dark_one = self._dark_one,
            bg_color = self._btn_color,
            bg_color_hover = self._btn_color_hover,
            bg_color_pressed = self._btn_color_pressed,
            icon_color = self._icon_color,
            icon_color_hover = self._icon_color_hover,
            icon_color_pressed = self._icon_color_pressed,
            icon_color_active = self._icon_color_pressed,
            context_color = self._context_color,
            text_foreground = self._text_title_color,
            icon_path = self._icon_close_path,
            radius = 6,
        )
        self.btn_close.setParent(self.btn_frame)
        self.btn_close.setObjectName("btn_close_left_column")

        # ADD TO TITLE LAYOUT
        self.title_bg_layout.addWidget(self.icon_frame)
        self.title_bg_layout.addWidget(self.title_label)
        self.title_bg_layout.addWidget(self.btn_frame)

        # ADD TITLE BG TO LAYOUT
        self.title_base_layout.addWidget(self.title_bg_frame)

        # CONTENT FRAME
        # ///////////////////////////////////////////////////////////////
        self.content_frame = QFrame()
        self.content_frame.setStyleSheet("background: none")

        # ADD TO LAYOUT
        # ///////////////////////////////////////////////////////////////
        self.base_layout.addWidget(self.title_frame)
        self.base_layout.addWidget(self.content_frame)



# %%
from pylab import np, plt
plt.style.use('ggplot')

I_AMPL = 5.0
PERIOD = 0.1 # s
CL_TS = 1e-4
time = np.arange(0, PERIOD, CL_TS)
phase_angle = np.arange(-np.pi, np.pi, 2*np.pi / len(time))
phase_current = np.sin(phase_angle) * I_AMPL


""" By Current Value """
def sigmoid(x, a1=0, a2=5, a3=10):
    return a1 * x + a2 / (1.0 + np.exp(-a3 * x)) - a2*0.5

bool_printed = False
def quadratic(current, I_turning, V_turning, I_plateau, V_plateau):
    slope = (V_plateau - V_turning) / (I_plateau - I_turning)
    a1 = 2*V_turning/I_turning - slope
    a2 = (V_turning - I_turning*slope)/I_turning**2
    global bool_printed
    if not bool_printed:
        print(a1, a2)
        bool_printed = True
    abs_current = abs(current)
    if abs_current < I_turning:
        return np.sign(current)*(a1*abs_current - a2*abs_current**2) # nonlinear
    elif abs_current < I_plateau:
        # V_turning = a1*current - a2*current**2
        # slope = a1 - 2*a2*I_turning
        return np.sign(current)*(V_turning + slope*(abs_current-I_turning))
    else:
        return np.sign(current)*V_plateau

# u-i curve quadratic
RATIO_TURNING_1 = 0.8
RATIO_TURNING_0 = 0.9
uic_q = [quadratic(current_value, 
                        I_turning= 2*RATIO_TURNING_0*RATIO_TURNING_1,
                        V_turning=10*RATIO_TURNING_0,
                        I_plateau= 2, 
                        V_plateau=10
                    ) for current_value in phase_current]
fig_UIcurve = plt.figure(figsize=(20,6))
fig_UIcurve.gca().plot(phase_current, uic_q)

fig_timeDomain = plt.figure(figsize=(20,6))
fig_timeDomain.gca().plot(time, uic_q)
fig_timeDomain.gca().plot(time, phase_current)

# %%
import sys
sys.path.insert(1, r'D:\DrH\Codes\visualize')
from DFT_CJH import myDFT_single_channel, DFT_CJH

ax = plt.figure(figsize=(20,6)).gca()

REPEAT = 10
time = np.arange(0, REPEAT*PERIOD, CL_TS)

signal = np.array(REPEAT*list(uic_q))
DFT_CJH(ax, time, signal, bool_use_dB=False)

ax.set_xlim([0,200])
# ax.set_ylim([-50,5])

print(len(time), len(signal))

# %%
REPEAT = 1
time = np.arange(0, REPEAT*PERIOD, CL_TS)

harmonics = dict()
harmonics[1] = []
harmonics[5] = []
harmonics[7] = []
harmonics[11] = []
harmonics[13] = []

# Free parameters
OPTION = 1
if OPTION == 1: # I_plateau is dependent parameter
    LIST_PARAM = LIST_I_Plateau = np.arange(0.1, 2, 0.05)

fig_UIcurve = plt.figure(figsize=(20,6))
fig_timeDomain = plt.figure(figsize=(20,6))

for I_plateau in LIST_I_Plateau:

    # Dependent parameters
    if OPTION == 1:
        pass

    uic_q = [quadratic(current_value, 
                        I_turning= 2*RATIO_TURNING_0*RATIO_TURNING_1,
                        V_turning=10*RATIO_TURNING_0,
                        I_plateau= I_plateau, 
                        V_plateau=10
                    ) for current_value in phase_current]

    fig_UIcurve.gca().plot(phase_current, uic_q)
    fig_timeDomain.gca().plot(time, uic_q)

    signal = np.array(REPEAT*list(uic_q))
    dft_dict = myDFT_single_channel(time, signal)

    # print([el for el in dft_dict['ampl'][0:15]])
    # print([el for el in dft_dict['ampl'][0:15] if el>1e-5])

    if REPEAT == 1: # 如果REPEAT变了分辨率就变了，间隔也就变了哦。
        harmonics[ 1].append(dft_dict['ampl'][ 1]); #print(' 10 Hz', dft_dict['ampl'][ 1])
        harmonics[ 5].append(dft_dict['ampl'][ 5]); #print(' 50 Hz', dft_dict['ampl'][ 5])
        harmonics[ 7].append(dft_dict['ampl'][ 7]); #print(' 70 Hz', dft_dict['ampl'][ 7])
        harmonics[11].append(dft_dict['ampl'][11]); #print('110 Hz', dft_dict['ampl'][11])
        harmonics[13].append(dft_dict['ampl'][13]); #print('130 Hz', dft_dict['ampl'][13])

# %%
plt.figure(figsize=(20,6))
plt.plot(harmonics[ 1], '.-')
plt.figure(figsize=(20,6))
plt.plot(harmonics[ 5], '.-')
plt.plot(harmonics[ 7], '.-')
plt.plot(harmonics[11], '.-')
plt.plot(harmonics[13], '.-')
plt.show()

# %%



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

style = """
/* HORIZONTAL */
QSlider {{ margin: {_margin}px; }}
QSlider::groove:horizontal {{
    border-radius: {_bg_radius}px;
    height: {_bg_size}px;
	margin: 0px;
	background-color: {_bg_color};
}}
QSlider::groove:horizontal:hover {{ background-color: {_bg_color_hover}; }}
QSlider::handle:horizontal {{
    border: none;
    height: {_handle_size}px;
    width: {_handle_size}px;
    margin: {_handle_margin}px;
	border-radius: {_handle_radius}px;
    background-color: {_handle_color};
}}
QSlider::handle:horizontal:hover {{ background-color: {_handle_color_hover}; }}
QSlider::handle:horizontal:pressed {{ background-color: {_handle_color_pressed}; }}

/* VERTICAL */
QSlider::groove:vertical {{
    border-radius: {_bg_radius}px;
    width: {_bg_size}px;
    margin: 0px;
	background-color: {_bg_color};
}}
QSlider::groove:vertical:hover {{ background-color: {_bg_color_hover}; }}
QSlider::handle:vertical {{
	border: none;
    height: {_handle_size}px;
    width: {_handle_size}px;
    margin: {_handle_margin}px;
	border-radius: {_handle_radius}px;
    background-color: {_handle_color};
}}
QSlider::handle:vertical:hover {{ background-color: {_handle_color_hover}; }}
QSlider::handle:vertical:pressed {{ background-color: {_handle_color_pressed}; }}
"""

class PySlider(QSlider):
    def __init__(
        self,
        margin = 0,
        bg_size = 20,
        bg_radius = 10,
        bg_color = "#1b1e23",
        bg_color_hover = "#1e2229",
        handle_margin = 2,
        handle_size = 16,
        handle_radius = 8,
        handle_color = "#568af2",
        handle_color_hover = "#6c99f4",
        handle_color_pressed = "#3f6fd1"
    ):
        super(PySlider, self).__init__()

        # FORMAT STYLE
        # ///////////////////////////////////////////////////////////////
        adjust_style = style.format(
            _margin = margin,
            _bg_size = bg_size,
            _bg_radius = bg_radius,
            _bg_color = bg_color,
            _bg_color_hover = bg_color_hover,
            _handle_margin = handle_margin,
            _handle_size = handle_size,
            _handle_radius = handle_radius,
            _handle_color = handle_color,
            _handle_color_hover = handle_color_hover,
            _handle_color_pressed = handle_color_pressed
        )

        # APPLY CUSTOM STYLE
        # ///////////////////////////////////////////////////////////////
        self.setStyleSheet(adjust_style)


# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# CUSTOM LEFT MENU
# ///////////////////////////////////////////////////////////////
class PyDiv(QWidget):
    def __init__(self, color):
        super().__init__()

        self.layout = QHBoxLayout(self)
        self.layout.setContentsMargins(5,0,5,0)
        self.frame_line = QFrame()
        self.frame_line.setStyleSheet(f"background: {color};")
        self.frame_line.setMaximumHeight(1)
        self.frame_line.setMinimumHeight(1)
        self.layout.addWidget(self.frame_line)
        self.setMaximumHeight(1)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT PACKAGES AND MODULES
# ///////////////////////////////////////////////////////////////
from gui.core.functions import Functions

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

# IMPORT SETTINGS
# ///////////////////////////////////////////////////////////////
from gui.core.json_settings import Settings

# IMPORT THEME COLORS
# ///////////////////////////////////////////////////////////////
from gui.core.json_themes import Themes

# IMPORT PY ONE DARK WIDGETS
# ///////////////////////////////////////////////////////////////
from gui.widgets import *

# IMPORT SETUP MAIN WINDOW
# ///////////////////////////////////////////////////////////////
from . setup_main_window import *

# IMPORT MAIN WINDOW PAGES / AND SIDE BOXES FOR APP
# ///////////////////////////////////////////////////////////////
from gui.uis.pages.ui_main_pages import Ui_MainPages

# RIGHT COLUMN
# ///////////////////////////////////////////////////////////////
from gui.uis.columns.ui_right_column import Ui_RightColumn

# CREDITS
# ///////////////////////////////////////////////////////////////
from gui.widgets.py_credits_bar.py_credits import PyCredits

# PY WINDOW
# ///////////////////////////////////////////////////////////////
class UI_MainWindow(object):
    def setup_ui(self, parent):
        if not parent.objectName():
            parent.setObjectName("MainWindow")

        # LOAD SETTINGS
        # ///////////////////////////////////////////////////////////////
        settings = Settings()
        self.settings = settings.items

        # LOAD THEME COLOR
        # ///////////////////////////////////////////////////////////////
        themes = Themes()
        self.themes = themes.items

        # SET INITIAL PARAMETERS
        parent.resize(self.settings["startup_size"][0], self.settings["startup_size"][1])
        parent.setMinimumSize(self.settings["minimum_size"][0], self.settings["minimum_size"][1])

        # SET CENTRAL WIDGET
        # Add central widget to app
        # ///////////////////////////////////////////////////////////////
        self.central_widget = QWidget()
        self.central_widget.setStyleSheet(f'''
            font: {self.settings["font"]["text_size"]}pt "{self.settings["font"]["family"]}";
            color: {self.themes["app_color"]["text_foreground"]};
        ''')
        self.central_widget_layout = QVBoxLayout(self.central_widget)
        if self.settings["custom_title_bar"]:
            self.central_widget_layout.setContentsMargins(10,10,10,10)
        else:
            self.central_widget_layout.setContentsMargins(0,0,0,0)
        
        # LOAD PY WINDOW CUSTOM WIDGET
        # Add inside PyWindow "layout" all Widgets
        # ///////////////////////////////////////////////////////////////
        self.window = PyWindow(
            parent,
            bg_color = self.themes["app_color"]["bg_one"],
            border_color = self.themes["app_color"]["bg_two"],
            text_color = self.themes["app_color"]["text_foreground"]
        )
        
        # If disable custom title bar
        if not self.settings["custom_title_bar"]:
            self.window.set_stylesheet(border_radius = 0, border_size = 0)
        
        # ADD PY WINDOW TO CENTRAL WIDGET
        self.central_widget_layout.addWidget(self.window)

        # ADD FRAME LEFT MENU
        # Add here the custom left menu bar
        # ///////////////////////////////////////////////////////////////
        left_menu_margin = self.settings["left_menu_content_margins"]
        left_menu_minimum = self.settings["lef_menu_size"]["minimum"]
        self.left_menu_frame = QFrame()
        self.left_menu_frame.setMaximumSize(left_menu_minimum + (left_menu_margin * 2), 17280)
        self.left_menu_frame.setMinimumSize(left_menu_minimum + (left_menu_margin * 2), 0)

        # LEFT MENU LAYOUT
        self.left_menu_layout = QHBoxLayout(self.left_menu_frame)
        self.left_menu_layout.setContentsMargins(
            left_menu_margin,
            left_menu_margin,
            left_menu_margin,
            left_menu_margin
        )

        # ADD LEFT MENU
        # Add custom left menu here
        # ///////////////////////////////////////////////////////////////
        self.left_menu = PyLeftMenu(
            parent = self.left_menu_frame,
            app_parent = self.central_widget, # For tooltip parent
            dark_one = self.themes["app_color"]["dark_one"],
            dark_three = self.themes["app_color"]["dark_three"],
            dark_four = self.themes["app_color"]["dark_four"],
            bg_one = self.themes["app_color"]["bg_one"],
            icon_color = self.themes["app_color"]["icon_color"],
            icon_color_hover = self.themes["app_color"]["icon_hover"],
            icon_color_pressed = self.themes["app_color"]["icon_pressed"],
            icon_color_active = self.themes["app_color"]["icon_active"],
            context_color = self.themes["app_color"]["context_color"],
            text_foreground = self.themes["app_color"]["text_foreground"],
            text_active = self.themes["app_color"]["text_active"]
        )
        self.left_menu_layout.addWidget(self.left_menu)

        # ADD LEFT COLUMN
        # Add here the left column with Stacked Widgets
        # ///////////////////////////////////////////////////////////////
        self.left_column_frame = QFrame()
        self.left_column_frame.setMaximumWidth(self.settings["left_column_size"]["minimum"])
        self.left_column_frame.setMinimumWidth(self.settings["left_column_size"]["minimum"])
        self.left_column_frame.setStyleSheet(f"background: {self.themes['app_color']['bg_two']}")

        # ADD LAYOUT TO LEFT COLUMN
        self.left_column_layout = QVBoxLayout(self.left_column_frame)
        self.left_column_layout.setContentsMargins(0,0,0,0)

        # ADD CUSTOM LEFT MENU WIDGET
        self.left_column = PyLeftColumn(
            parent,
            app_parent = self.central_widget,
            text_title = "Settings Left Frame",
            text_title_size = self.settings["font"]["title_size"],
            text_title_color = self.themes['app_color']['text_foreground'],
            icon_path = Functions.set_svg_icon("icon_settings.svg"),
            dark_one = self.themes['app_color']['dark_one'],
            bg_color = self.themes['app_color']['bg_three'],
            btn_color = self.themes['app_color']['bg_three'],
            btn_color_hover = self.themes['app_color']['bg_two'],
            btn_color_pressed = self.themes['app_color']['bg_one'],
            icon_color = self.themes['app_color']['icon_color'],
            icon_color_hover = self.themes['app_color']['icon_hover'],
            context_color = self.themes['app_color']['context_color'],
            icon_color_pressed = self.themes['app_color']['icon_pressed'],
            icon_close_path = Functions.set_svg_icon("icon_close.svg")
        )
        self.left_column_layout.addWidget(self.left_column)

        # ADD RIGHT WIDGETS
        # Add here the right widgets
        # ///////////////////////////////////////////////////////////////
        self.right_app_frame = QFrame()

        # ADD RIGHT APP LAYOUT
        self.right_app_layout = QVBoxLayout(self.right_app_frame)
        self.right_app_layout.setContentsMargins(3,3,3,3)
        self.right_app_layout.setSpacing(6)

        # ADD TITLE BAR FRAME
        # ///////////////////////////////////////////////////////////////
        self.title_bar_frame = QFrame()
        self.title_bar_frame.setMinimumHeight(40)
        self.title_bar_frame.setMaximumHeight(40)
        self.title_bar_layout = QVBoxLayout(self.title_bar_frame)
        self.title_bar_layout.setContentsMargins(0,0,0,0)
        
        # ADD CUSTOM TITLE BAR TO LAYOUT
        self.title_bar = PyTitleBar(
            parent,
            logo_width = 100,
            app_parent = self.central_widget,
            logo_image = "logo_top_100x22.svg",
            bg_color = self.themes["app_color"]["bg_two"],
            div_color = self.themes["app_color"]["bg_three"],
            btn_bg_color = self.themes["app_color"]["bg_two"],
            btn_bg_color_hover = self.themes["app_color"]["bg_three"],
            btn_bg_color_pressed = self.themes["app_color"]["bg_one"],
            icon_color = self.themes["app_color"]["icon_color"],
            icon_color_hover = self.themes["app_color"]["icon_hover"],
            icon_color_pressed = self.themes["app_color"]["icon_pressed"],
            icon_color_active = self.themes["app_color"]["icon_active"],
            context_color = self.themes["app_color"]["context_color"],
            dark_one = self.themes["app_color"]["dark_one"],
            text_foreground = self.themes["app_color"]["text_foreground"],
            radius = 8,
            font_family = self.settings["font"]["family"],
            title_size = self.settings["font"]["title_size"],
            is_custom_title_bar = self.settings["custom_title_bar"]
        )
        self.title_bar_layout.addWidget(self.title_bar)

        # ADD CONTENT AREA
        # ///////////////////////////////////////////////////////////////
        self.content_area_frame = QFrame()

        # CREATE LAYOUT
        self.content_area_layout = QHBoxLayout(self.content_area_frame)
        self.content_area_layout.setContentsMargins(0,0,0,0)
        self.content_area_layout.setSpacing(0)

        # LEFT CONTENT
        self.content_area_left_frame = QFrame()

        # IMPORT MAIN PAGES TO CONTENT AREA
        self.load_pages = Ui_MainPages()
        self.load_pages.setupUi(self.content_area_left_frame)

        # RIGHT BAR
        self.right_column_frame = QFrame()
        self.right_column_frame.setMinimumWidth(self.settings["right_column_size"]["minimum"])
        self.right_column_frame.setMaximumWidth(self.settings["right_column_size"]["minimum"])

        # IMPORT RIGHT COLUMN
        # ///////////////////////////////////////////////////////////////
        self.content_area_right_layout = QVBoxLayout(self.right_column_frame)
        self.content_area_right_layout.setContentsMargins(5,5,5,5)
        self.content_area_right_layout.setSpacing(0)

        # RIGHT BG
        self.content_area_right_bg_frame = QFrame()
        self.content_area_right_bg_frame.setObjectName("content_area_right_bg_frame")
        self.content_area_right_bg_frame.setStyleSheet(f'''
        #content_area_right_bg_frame {{
            border-radius: 8px;
            background-color: {self.themes["app_color"]["bg_two"]};
        }}
        ''')

        # ADD BG
        self.content_area_right_layout.addWidget(self.content_area_right_bg_frame)


        # ADD RIGHT PAGES TO RIGHT COLUMN
        self.right_column = Ui_RightColumn()
        self.right_column.setupUi(self.content_area_right_bg_frame)

        # ADD TO LAYOUTS
        self.content_area_layout.addWidget(self.content_area_left_frame)
        self.content_area_layout.addWidget(self.right_column_frame)

        # CREDITS / BOTTOM APP FRAME
        # ///////////////////////////////////////////////////////////////
        self.credits_frame = QFrame()
        self.credits_frame.setMinimumHeight(26)
        self.credits_frame.setMaximumHeight(26)

        # CREATE LAYOUT
        self.credits_layout = QVBoxLayout(self.credits_frame)
        self.credits_layout.setContentsMargins(0,0,0,0)

        # ADD CUSTOM WIDGET CREDITS
        self.credits = PyCredits(
            bg_two = self.themes["app_color"]["bg_two"],
            copyright = self.settings["copyright"],
            version = self.settings["version"],
            font_family = self.settings["font"]["family"],
            text_size = self.settings["font"]["text_size"],
            text_description_color = self.themes["app_color"]["text_description"]
        )

        #  ADD TO LAYOUT
        self.credits_layout.addWidget(self.credits)

        # ADD WIDGETS TO RIGHT LAYOUT
        # ///////////////////////////////////////////////////////////////
        self.right_app_layout.addWidget(self.title_bar_frame)
        self.right_app_layout.addWidget(self.content_area_frame)
        self.right_app_layout.addWidget(self.credits_frame)
        
        # ADD WIDGETS TO "PyWindow"
        # Add here your custom widgets or default widgets
        # ///////////////////////////////////////////////////////////////
        self.window.layout.addWidget(self.left_menu_frame)
        self.window.layout.addWidget(self.left_column_frame)
        self.window.layout.addWidget(self.right_app_frame)

        # ADD CENTRAL WIDGET AND SET CONTENT MARGINS
        # ///////////////////////////////////////////////////////////////
        parent.setCentralWidget(self.central_widget)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from qt_core import *

class PyToggle(QCheckBox):
    def __init__(
        self,
        width = 50,
        bg_color = "#777", 
        circle_color = "#DDD",
        active_color = "#00BCFF",
        animation_curve = QEasingCurve.OutBounce
    ):
        QCheckBox.__init__(self)
        self.setFixedSize(width, 28)
        self.setCursor(Qt.PointingHandCursor)

        # COLORS
        self._bg_color = bg_color
        self._circle_color = circle_color
        self._active_color = active_color

        self._position = 3
        self.animation = QPropertyAnimation(self, b"position")
        self.animation.setEasingCurve(animation_curve)
        self.animation.setDuration(500)
        self.stateChanged.connect(self.setup_animation)

    @Property(float)
    def position(self):
        return self._position

    @position.setter
    def position(self, pos):
        self._position = pos
        self.update()

    # START STOP ANIMATION
    def setup_animation(self, value):
        self.animation.stop()
        if value:
            self.animation.setEndValue(self.width() - 26)
        else:
            self.animation.setEndValue(4)
        self.animation.start()
    
    def hitButton(self, pos: QPoint):
        return self.contentsRect().contains(pos)

    def paintEvent(self, e):
        p = QPainter(self)
        p.setRenderHint(QPainter.Antialiasing)
        p.setFont(QFont("Segoe UI", 9))

        # SET PEN
        p.setPen(Qt.NoPen)

        # DRAW RECT
        rect = QRect(0, 0, self.width(), self.height())        

        if not self.isChecked():
            p.setBrush(QColor(self._bg_color))
            p.drawRoundedRect(0,0,rect.width(), 28, 14, 14)
            p.setBrush(QColor(self._circle_color))
            p.drawEllipse(self._position, 3, 22, 22)
        else:
            p.setBrush(QColor(self._active_color))
            p.drawRoundedRect(0,0,rect.width(), 28, 14, 14)
            p.setBrush(QColor(self._circle_color))
            p.drawEllipse(self._position, 3, 22, 22)

        p.end()